import re
import json
import random
import string
import base64
import asyncio
from datetime import datetime

from .log_config import logger

from .utils import safe_get, generate_sse_response, generate_no_stream_response, end_of_line, parse_json_safely, upload_image_to_0x0st

async def check_response(response, error_log):
    if response and not (200 <= response.status_code < 300):
        error_message = await response.aread()
        error_str = error_message.decode('utf-8', errors='replace')
        try:
            error_json = await asyncio.to_thread(json.loads, error_str)
        except json.JSONDecodeError:
            error_json = error_str
        return {"error": f"{error_log} HTTP Error", "status_code": response.status_code, "details": error_json}
    return None

async def gemini_json_poccess(response_json):
    promptTokenCount = 0
    candidatesTokenCount = 0
    totalTokenCount = 0
    image_base64 = None

    json_data = safe_get(response_json, "candidates", 0, "content", default=None)
    finishReason = safe_get(response_json, "candidates", 0 , "finishReason", default=None)
    if finishReason:
        promptTokenCount = safe_get(response_json, "usageMetadata", "promptTokenCount", default=0)
        candidatesTokenCount = safe_get(response_json, "usageMetadata", "candidatesTokenCount", default=0)
        totalTokenCount = safe_get(response_json, "usageMetadata", "totalTokenCount", default=0)
        if finishReason != "STOP":
            logger.error(f"finishReason: {finishReason}")

    content = reasoning_content = safe_get(json_data, "parts", 0, "text", default="")
    b64_json = safe_get(json_data, "parts", 0, "inlineData", "data", default="")
    if b64_json:
        image_base64 = b64_json

    is_thinking = safe_get(json_data, "parts", 0, "thought", default=False)
    if is_thinking:
        content = safe_get(json_data, "parts", 1, "text", default="")

    function_call_name = safe_get(json_data, "parts", 0, "functionCall", "name", default=None)
    function_full_response = safe_get(json_data, "parts", 0, "functionCall", "args", default="")
    function_full_response = await asyncio.to_thread(json.dumps, function_full_response) if function_full_response else None

    blockReason = safe_get(json_data, 0, "promptFeedback", "blockReason", default=None)

    return is_thinking, reasoning_content, content, image_base64, function_call_name, function_full_response, finishReason, blockReason, promptTokenCount, candidatesTokenCount, totalTokenCount

async def fetch_gemini_response_stream(client, url, headers, payload, model, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_gemini_response_stream")
        if error_message:
            yield error_message
            return
        buffer = ""
        promptTokenCount = 0
        candidatesTokenCount = 0
        totalTokenCount = 0
        parts_json = ""
        async for chunk in response.aiter_text():
            buffer += chunk
            if buffer and "\n" not in buffer:
                buffer += "\n"

            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                if line.startswith("data: "):
                    parts_json = line.lstrip("data: ").strip()
                    try:
                        response_json = await asyncio.to_thread(json.loads, parts_json)
                    except json.JSONDecodeError:
                        # logger.error(f"JSON decode error: {parts_json}")
                        continue
                else:
                    parts_json += line
                    parts_json = parts_json.lstrip("[,")
                    try:
                        response_json = await asyncio.to_thread(json.loads, parts_json)
                    except json.JSONDecodeError:
                        continue

                # https://ai.google.dev/api/generate-content?hl=zh-cn#FinishReason
                is_thinking, reasoning_content, content, image_base64, function_call_name, function_full_response, finishReason, blockReason, promptTokenCount, candidatesTokenCount, totalTokenCount = await gemini_json_poccess(response_json)

                if is_thinking:
                    sse_string = await generate_sse_response(timestamp, model, reasoning_content=reasoning_content)
                    yield sse_string
                if not image_base64 and content:
                    sse_string = await generate_sse_response(timestamp, model, content=content)
                    yield sse_string

                if image_base64:
                    if "gemini-2.5-flash-image" not in model:
                        yield await generate_no_stream_response(timestamp, model, content=content, tools_id=None, function_call_name=None, function_call_content=None, role=None, total_tokens=totalTokenCount, prompt_tokens=promptTokenCount, completion_tokens=candidatesTokenCount, image_base64=image_base64)
                    else:
                        image_url = await upload_image_to_0x0st("data:image/png;base64," + image_base64)
                        sse_string = await generate_sse_response(timestamp, model, content=f"\n\n![image]({image_url})")
                        yield sse_string

                if function_call_name:
                    sse_string = await generate_sse_response(timestamp, model, content=None, tools_id="chatcmpl-9inWv0yEtgn873CxMBzHeCeiHctTV", function_call_name=function_call_name)
                    yield sse_string
                if function_full_response:
                    sse_string = await generate_sse_response(timestamp, model, content=None, tools_id="chatcmpl-9inWv0yEtgn873CxMBzHeCeiHctTV", function_call_name=None, function_call_content=function_full_response)
                    yield sse_string

                if parts_json == "[]" or blockReason == "PROHIBITED_CONTENT":
                    sse_string = await generate_sse_response(timestamp, model, stop="PROHIBITED_CONTENT")
                    yield sse_string
                elif finishReason:
                    sse_string = await generate_sse_response(timestamp, model, stop="stop")
                    yield sse_string
                    break

                parts_json = ""

        sse_string = await generate_sse_response(timestamp, model, None, None, None, None, None, totalTokenCount, promptTokenCount, candidatesTokenCount)
        yield sse_string

    yield "data: [DONE]" + end_of_line

async def fetch_vertex_claude_response_stream(client, url, headers, payload, model, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_vertex_claude_response_stream")
        if error_message:
            yield error_message
            return

        buffer = ""
        revicing_function_call = False
        function_full_response = "{"
        need_function_call = False
        is_finish = False
        promptTokenCount = 0
        candidatesTokenCount = 0
        totalTokenCount = 0

        async for chunk in response.aiter_text():
            buffer += chunk
            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                # logger.info(f"{line}")

                if line and '\"finishReason\": \"' in line:
                    is_finish = True
                if is_finish and '\"promptTokenCount\": ' in line:
                    json_data = parse_json_safely( "{" + line + "}")
                    promptTokenCount = json_data.get('promptTokenCount', 0)
                if is_finish and '\"candidatesTokenCount\": ' in line:
                    json_data = parse_json_safely( "{" + line + "}")
                    candidatesTokenCount = json_data.get('candidatesTokenCount', 0)
                if is_finish and '\"totalTokenCount\": ' in line:
                    json_data = parse_json_safely( "{" + line + "}")
                    totalTokenCount = json_data.get('totalTokenCount', 0)

                if line and '\"text\": \"' in line and is_finish == False:
                    try:
                        json_data = await asyncio.to_thread(json.loads, "{" + line.strip().rstrip(",") + "}")
                        content = json_data.get('text', '')
                        sse_string = await generate_sse_response(timestamp, model, content=content)
                        yield sse_string
                    except json.JSONDecodeError:
                        logger.error(f"无法解析JSON: {line}")

                if line and ('\"type\": \"tool_use\"' in line or revicing_function_call):
                    revicing_function_call = True
                    need_function_call = True
                    if ']' in line:
                        revicing_function_call = False
                        continue

                    function_full_response += line

        if need_function_call:
            function_call = await asyncio.to_thread(json.loads, function_full_response)
            function_call_name = function_call["name"]
            function_call_id = function_call["id"]
            sse_string = await generate_sse_response(timestamp, model, content=None, tools_id=function_call_id, function_call_name=function_call_name)
            yield sse_string
            function_full_response = await asyncio.to_thread(json.dumps, function_call["input"])
            sse_string = await generate_sse_response(timestamp, model, content=None, tools_id=function_call_id, function_call_name=None, function_call_content=function_full_response)
            yield sse_string

        sse_string = await generate_sse_response(timestamp, model, None, None, None, None, None, totalTokenCount, promptTokenCount, candidatesTokenCount)
        yield sse_string

    yield "data: [DONE]" + end_of_line

async def fetch_gpt_response_stream(client, url, headers, payload, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    random.seed(timestamp)
    random_str = ''.join(random.choices(string.ascii_letters + string.digits, k=29))
    is_thinking = False
    has_send_thinking = False
    ark_tag = False
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_gpt_response_stream")
        if error_message:
            yield error_message
            return

        buffer = ""
        enter_buffer = ""

        input_tokens = 0
        output_tokens = 0

        async for chunk in response.aiter_text():
            buffer += chunk
            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                # logger.info("line: %s", repr(line))
                if line.startswith(": keepalive"):
                    yield line + end_of_line
                    continue
                if line and not line.startswith(":") and (result:=line.lstrip("data: ").strip()) and not line.startswith("event: "):
                    if result.strip() == "[DONE]":
                        break
                    line = await asyncio.to_thread(json.loads, result)
                    line['id'] = f"chatcmpl-{random_str}"

                    # v1/responses
                    if line.get("type") == "response.reasoning_summary_text.delta" and line.get("delta"):
                        sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=line.get("delta"))
                        yield sse_string
                        continue
                    elif line.get("type") == "response.output_text.delta" and line.get("delta"):
                        sse_string = await generate_sse_response(timestamp, payload["model"], content=line.get("delta"))
                        yield sse_string
                        continue
                    elif line.get("type") == "response.output_text.done":
                        sse_string = await generate_sse_response(timestamp, payload["model"], stop="stop")
                        yield sse_string
                        continue
                    elif line.get("type") == "response.completed":
                        input_tokens = safe_get(line, "response", "usage", "input_tokens", default=0)
                        output_tokens = safe_get(line, "response", "usage", "output_tokens", default=0)
                        continue
                    elif line.get("type", "").startswith("response."):
                        continue

                    # 处理 <think> 标签
                    content = safe_get(line, "choices", 0, "delta", "content", default="")
                    if "<think>" in content:
                        is_thinking = True
                        ark_tag = True
                        content = content.replace("<think>", "")
                    if "</think>" in content:
                        end_think_reasoning_content = ""
                        end_think_content = ""
                        is_thinking = False

                        if content.rstrip('\n').endswith("</think>"):
                            end_think_reasoning_content = content.replace("</think>", "").rstrip('\n')
                        elif content.lstrip('\n').startswith("</think>"):
                            end_think_content = content.replace("</think>", "").lstrip('\n')
                        else:
                            end_think_reasoning_content = content.split("</think>")[0]
                            end_think_content = content.split("</think>")[1]

                        if end_think_reasoning_content:
                            sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=end_think_reasoning_content)
                            yield sse_string
                        if end_think_content:
                            sse_string = await generate_sse_response(timestamp, payload["model"], content=end_think_content)
                            yield sse_string
                        continue
                    if is_thinking and ark_tag:
                        if not has_send_thinking:
                            content = content.replace("\n\n", "")
                        if content:
                            sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=content)
                            yield sse_string
                            has_send_thinking = True
                        continue

                    # 处理 poe thinking 标签
                    if "Thinking..." in content and "\n> " in content:
                        is_thinking = True
                        content = content.replace("Thinking...", "").replace("\n> ", "")
                    if is_thinking and "\n\n" in content and not ark_tag:
                        is_thinking = False
                    if is_thinking and not ark_tag:
                        content = content.replace("\n> ", "")
                        if not has_send_thinking:
                            content = content.replace("\n", "")
                        if content:
                            sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=content)
                            yield sse_string
                            has_send_thinking = True
                        continue

                    no_stream_content = safe_get(line, "choices", 0, "message", "content", default=None)
                    openrouter_reasoning = safe_get(line, "choices", 0, "delta", "reasoning", default="")
                    openrouter_base64_image = safe_get(line, "choices", 0, "delta", "images", 0, "image_url", "url", default="")
                    if openrouter_base64_image:
                        image_url = await upload_image_to_0x0st(openrouter_base64_image)
                        sse_string = await generate_sse_response(timestamp, payload["model"], content=f"\n\n![image]({image_url})")
                        yield sse_string
                        continue
                    azure_databricks_claude_summary_content = safe_get(line, "choices", 0, "delta", "content", 0, "summary", 0, "text", default="")
                    azure_databricks_claude_signature_content = safe_get(line, "choices", 0, "delta", "content", 0, "summary", 0, "signature", default="")
                    # print("openrouter_reasoning", repr(openrouter_reasoning), openrouter_reasoning.endswith("\\\\"), openrouter_reasoning.endswith("\\"))
                    if azure_databricks_claude_signature_content:
                        pass
                    elif azure_databricks_claude_summary_content:
                        sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=azure_databricks_claude_summary_content)
                        yield sse_string
                    elif openrouter_reasoning:
                        if openrouter_reasoning.endswith("\\"):
                            enter_buffer += openrouter_reasoning
                            continue
                        elif enter_buffer.endswith("\\") and openrouter_reasoning == 'n':
                            enter_buffer += "n"
                            continue
                        elif enter_buffer.endswith("\\n") and openrouter_reasoning == '\\n':
                            enter_buffer += "\\n"
                            continue
                        elif enter_buffer.endswith("\\n\\n"):
                            openrouter_reasoning = '\n\n' + openrouter_reasoning
                            enter_buffer = ""
                        elif enter_buffer:
                            openrouter_reasoning = enter_buffer + openrouter_reasoning
                            enter_buffer = ''
                        openrouter_reasoning = openrouter_reasoning.replace("\\n", "\n")

                        sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=openrouter_reasoning)
                        yield sse_string
                    elif no_stream_content and has_send_thinking == False:
                        sse_string = await generate_sse_response(safe_get(line, "created", default=None), safe_get(line, "model", default=None), content=no_stream_content)
                        yield sse_string
                    else:
                        if no_stream_content:
                            del line["choices"][0]["message"]
                        json_line = await asyncio.to_thread(json.dumps, line)
                        yield "data: " + json_line.strip() + end_of_line

    if input_tokens and output_tokens:
        sse_string = await generate_sse_response(timestamp, payload["model"], None, None, None, None, None, total_tokens=input_tokens + output_tokens, prompt_tokens=input_tokens, completion_tokens=output_tokens)
        yield sse_string

    yield "data: [DONE]" + end_of_line

async def fetch_azure_response_stream(client, url, headers, payload, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    is_thinking = False
    has_send_thinking = False
    ark_tag = False
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_azure_response_stream")
        if error_message:
            yield error_message
            return

        buffer = ""
        sse_string = ""
        async for chunk in response.aiter_text():
            buffer += chunk
            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                # logger.info("line: %s", repr(line))
                if line and not line.startswith(":") and (result:=line.lstrip("data: ").strip()):
                    if result.strip() == "[DONE]":
                        break
                    line = await asyncio.to_thread(json.loads, result)
                    no_stream_content = safe_get(line, "choices", 0, "message", "content", default="")
                    content = safe_get(line, "choices", 0, "delta", "content", default="")

                    # 处理 <think> 标签
                    if "<think>" in content:
                        is_thinking = True
                        ark_tag = True
                        content = content.replace("<think>", "")
                    if "</think>" in content:
                        is_thinking = False
                        content = content.replace("</think>", "")
                        if not content:
                            continue
                    if is_thinking and ark_tag:
                        if not has_send_thinking:
                            content = content.replace("\n\n", "")
                        if content:
                            sse_string = await generate_sse_response(timestamp, payload["model"], reasoning_content=content)
                            yield sse_string
                            has_send_thinking = True
                        continue

                    if no_stream_content or content or sse_string:
                        input_tokens = safe_get(line, "usage", "prompt_tokens", default=0)
                        output_tokens = safe_get(line, "usage", "completion_tokens", default=0)
                        total_tokens = safe_get(line, "usage", "total_tokens", default=0)
                        sse_string = await generate_sse_response(timestamp, safe_get(line, "model", default=None), content=no_stream_content or content, total_tokens=total_tokens, prompt_tokens=input_tokens, completion_tokens=output_tokens)
                        yield sse_string
                    else:
                        if no_stream_content:
                            del line["choices"][0]["message"]
                        json_line = await asyncio.to_thread(json.dumps, line)
                        yield "data: " + json_line.strip() + end_of_line
    yield "data: [DONE]" + end_of_line

async def fetch_cloudflare_response_stream(client, url, headers, payload, model, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_cloudflare_response_stream")
        if error_message:
            yield error_message
            return

        buffer = ""
        async for chunk in response.aiter_text():
            buffer += chunk
            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                # logger.info("line: %s", repr(line))
                if line.startswith("data:"):
                    line = line.lstrip("data: ")
                    if line == "[DONE]":
                        break
                    resp: dict = await asyncio.to_thread(json.loads, line)
                    message = resp.get("response")
                    if message:
                        sse_string = await generate_sse_response(timestamp, model, content=message)
                        yield sse_string
    yield "data: [DONE]" + end_of_line

async def fetch_cohere_response_stream(client, url, headers, payload, model, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_cohere_response_stream")
        if error_message:
            yield error_message
            return

        buffer = ""
        async for chunk in response.aiter_text():
            buffer += chunk
            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                # logger.info("line: %s", repr(line))
                resp: dict = await asyncio.to_thread(json.loads, line)
                if resp.get("is_finished") == True:
                    break
                if resp.get("event_type") == "text-generation":
                    message = resp.get("text")
                    sse_string = await generate_sse_response(timestamp, model, content=message)
                    yield sse_string
    yield "data: [DONE]" + end_of_line

async def fetch_claude_response_stream(client, url, headers, payload, model, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_claude_response_stream")
        if error_message:
            yield error_message
            return
        buffer = ""
        input_tokens = 0
        async for chunk in response.aiter_text():
            # logger.info(f"chunk: {repr(chunk)}")
            buffer += chunk
            while "\n" in buffer:
                line, buffer = buffer.split("\n", 1)
                # logger.info(line)

                if line.startswith("data:") and (line := line.lstrip("data: ")):
                    resp: dict = await asyncio.to_thread(json.loads, line)

                    input_tokens = input_tokens or safe_get(resp, "message", "usage", "input_tokens", default=0)
                    # cache_creation_input_tokens = safe_get(resp, "message", "usage", "cache_creation_input_tokens", default=0)
                    # cache_read_input_tokens = safe_get(resp, "message", "usage", "cache_read_input_tokens", default=0)
                    output_tokens = safe_get(resp, "usage", "output_tokens", default=0)
                    if output_tokens:
                        total_tokens = input_tokens + output_tokens
                        sse_string = await generate_sse_response(timestamp, model, None, None, None, None, None, total_tokens, input_tokens, output_tokens)
                        yield sse_string
                        break

                    text = safe_get(resp, "delta", "text", default="")
                    if text:
                        sse_string = await generate_sse_response(timestamp, model, text)
                        yield sse_string
                        continue

                    function_call_name = safe_get(resp, "content_block", "name", default=None)
                    tools_id = safe_get(resp, "content_block", "id", default=None)
                    if tools_id and function_call_name:
                        sse_string = await generate_sse_response(timestamp, model, None, tools_id, function_call_name, None)
                        yield sse_string

                    thinking_content = safe_get(resp, "delta", "thinking", default="")
                    if thinking_content:
                        sse_string = await generate_sse_response(timestamp, model, reasoning_content=thinking_content)
                        yield sse_string

                    function_call_content = safe_get(resp, "delta", "partial_json", default="")
                    if function_call_content:
                        sse_string = await generate_sse_response(timestamp, model, None, None, None, function_call_content)
                        yield sse_string

    yield "data: [DONE]" + end_of_line

async def fetch_aws_response_stream(client, url, headers, payload, model, timeout):
    timestamp = int(datetime.timestamp(datetime.now()))
    json_payload = await asyncio.to_thread(json.dumps, payload)
    async with client.stream('POST', url, headers=headers, content=json_payload, timeout=timeout) as response:
        error_message = await check_response(response, "fetch_aws_response_stream")
        if error_message:
            yield error_message
            return

        buffer = ""
        async for line in response.aiter_text():
            buffer += line
            while "\r" in buffer:
                line, buffer = buffer.split("\r", 1)
                if not line or \
                line.strip() == "" or \
                line.strip().startswith(':content-type') or \
                line.strip().startswith(':event-type'): # 过滤掉完全空的行或只有空白的行
                    continue

                json_match = re.search(r'event{.*?}', line)
                if not json_match:
                    continue
                try:
                    chunk_data = await asyncio.to_thread(json.loads, json_match.group(0).lstrip('event'))
                except json.JSONDecodeError:
                    logger.error(f"DEBUG json.JSONDecodeError: {json_match.group(0).lstrip('event')!r}")
                    continue

                # --- 后续处理逻辑不变 ---
                if "bytes" in chunk_data:
                    # 解码 Base64 编码的字节
                    decoded_bytes = base64.b64decode(chunk_data["bytes"])
                    # 将解码后的字节再次解析为 JSON
                    payload_chunk = await asyncio.to_thread(json.loads, decoded_bytes.decode('utf-8'))
                    # print(f"DEBUG payload_chunk: {payload_chunk!r}")

                    text = safe_get(payload_chunk, "delta", "text", default="")
                    if text:
                        sse_string = await generate_sse_response(timestamp, model, text, None, None)
                        yield sse_string

                    usage = safe_get(payload_chunk, "amazon-bedrock-invocationMetrics", default="")
                    if usage:
                        input_tokens = usage.get("inputTokenCount", 0)
                        output_tokens = usage.get("outputTokenCount", 0)
                        total_tokens = input_tokens + output_tokens
                        sse_string = await generate_sse_response(timestamp, model, None, None, None, None, None, total_tokens, input_tokens, output_tokens)
                        yield sse_string

    yield "data: [DONE]" + end_of_line

async def fetch_response(client, url, headers, payload, engine, model, timeout=200):
    response = None
    if payload.get("file"):
        file = payload.pop("file")
        response = await client.post(url, headers=headers, data=payload, files={"file": file}, timeout=timeout)
    else:
        json_payload = await asyncio.to_thread(json.dumps, payload)
        response = await client.post(url, headers=headers, content=json_payload, timeout=timeout)
    error_message = await check_response(response, "fetch_response")
    if error_message:
        yield error_message
        return

    if engine == "tts":
        yield response.read()

    elif engine == "gemini" or engine == "vertex-gemini" or engine == "aws":
        response_bytes = await response.aread()
        response_json = await asyncio.to_thread(json.loads, response_bytes)
        # print("response_json", json.dumps(response_json, indent=4, ensure_ascii=False))

        if isinstance(response_json, str):
            import ast
            parsed_data = ast.literal_eval(str(response_json))
        elif isinstance(response_json, list):
            parsed_data = response_json
        elif isinstance(response_json, dict):
            parsed_data = [response_json]
        else:
            logger.error(f"error fetch_response: Unknown response_json type: {type(response_json)}")
            parsed_data = response_json

        content = ""
        reasoning_content = ""
        image_base64 = ""
        parts_list = safe_get(parsed_data, 0, "candidates", 0, "content", "parts", default=[])
        for item in parts_list:
            chunk = safe_get(item, "text")
            b64_json = safe_get(item, "inlineData", "data", default="")
            if b64_json:
                image_base64 = b64_json
            is_think = safe_get(item, "thought", default=False)
            # logger.info(f"chunk: {repr(chunk)}")
            if chunk:
                if is_think:
                    reasoning_content += chunk
                else:
                    content += chunk

        usage_metadata = safe_get(parsed_data, -1, "usageMetadata")
        prompt_tokens = safe_get(usage_metadata, "promptTokenCount", default=0)
        candidates_tokens = safe_get(usage_metadata, "candidatesTokenCount", default=0)
        total_tokens = safe_get(usage_metadata, "totalTokenCount", default=0)

        role = safe_get(parsed_data, -1, "candidates", 0, "content", "role")
        if role == "model":
            role = "assistant"
        else:
            logger.error(f"Unknown role: {role}, parsed_data: {parsed_data}")
            role = "assistant"

        has_think = safe_get(parsed_data, 0, "candidates", 0, "content", "parts", 0, "thought", default=False)
        if has_think:
            function_message_parts_index = -1
        else:
            function_message_parts_index = 0
        function_call_name = safe_get(parsed_data, -1, "candidates", 0, "content", "parts", function_message_parts_index, "functionCall", "name", default=None)
        function_call_content = safe_get(parsed_data, -1, "candidates", 0, "content", "parts", function_message_parts_index, "functionCall", "args", default=None)

        timestamp = int(datetime.timestamp(datetime.now()))
        yield await generate_no_stream_response(timestamp, model, content=content, tools_id=None, function_call_name=function_call_name, function_call_content=function_call_content, role=role, total_tokens=total_tokens, prompt_tokens=prompt_tokens, completion_tokens=candidates_tokens, reasoning_content=reasoning_content, image_base64=image_base64)

    elif engine == "claude":
        response_bytes = await response.aread()
        response_json = await asyncio.to_thread(json.loads, response_bytes)
        # print("response_json", json.dumps(response_json, indent=4, ensure_ascii=False))

        content = safe_get(response_json, "content", 0, "text")

        prompt_tokens = safe_get(response_json, "usage", "input_tokens")
        output_tokens = safe_get(response_json, "usage", "output_tokens")
        total_tokens = prompt_tokens + output_tokens

        role = safe_get(response_json, "role")

        function_call_name = safe_get(response_json, "content", 1, "name", default=None)
        function_call_content = safe_get(response_json, "content", 1, "input", default=None)
        tools_id = safe_get(response_json, "content", 1, "id", default=None)

        timestamp = int(datetime.timestamp(datetime.now()))
        yield await generate_no_stream_response(timestamp, model, content=content, tools_id=tools_id, function_call_name=function_call_name, function_call_content=function_call_content, role=role, total_tokens=total_tokens, prompt_tokens=prompt_tokens, completion_tokens=output_tokens)

    elif engine == "azure":
        response_bytes = await response.aread()
        response_json = await asyncio.to_thread(json.loads, response_bytes)
        # 删除 content_filter_results
        if "choices" in response_json:
            for choice in response_json["choices"]:
                if "content_filter_results" in choice:
                    del choice["content_filter_results"]

        # 删除 prompt_filter_results
        if "prompt_filter_results" in response_json:
            del response_json["prompt_filter_results"]

        yield response_json

    elif "dashscope.aliyuncs.com" in url and "multimodal-generation" in url:
        response_bytes = await response.aread()
        response_json = await asyncio.to_thread(json.loads, response_bytes)
        content = safe_get(response_json, "output", "choices", 0, "message", "content", 0, default=None)
        yield content

    elif "embedContent" in url:
        response_bytes = await response.aread()
        response_json = await asyncio.to_thread(json.loads, response_bytes)
        content = safe_get(response_json, "embedding", "values", default=[])
        response_embedContent = {
            "object": "list",
            "data": [
                {
                    "object": "embedding",
                    "embedding":content,
                    "index": 0
                }
            ],
            "model": model,
            "usage": {
                "prompt_tokens": 0,
                "total_tokens": 0
            }
        }

        yield response_embedContent
    else:
        response_bytes = await response.aread()
        response_json = await asyncio.to_thread(json.loads, response_bytes)
        yield response_json

async def fetch_response_stream(client, url, headers, payload, engine, model, timeout=200):
    if engine == "gemini" or engine == "vertex-gemini":
        async for chunk in fetch_gemini_response_stream(client, url, headers, payload, model, timeout):
            yield chunk
    elif engine == "claude" or engine == "vertex-claude":
        async for chunk in fetch_claude_response_stream(client, url, headers, payload, model, timeout):
            yield chunk
    elif engine == "aws":
        async for chunk in fetch_aws_response_stream(client, url, headers, payload, model, timeout):
            yield chunk
    elif engine == "gpt" or engine == "openrouter" or engine == "azure-databricks":
        async for chunk in fetch_gpt_response_stream(client, url, headers, payload, timeout):
            yield chunk
    elif engine == "azure":
        async for chunk in fetch_azure_response_stream(client, url, headers, payload, timeout):
            yield chunk
    elif engine == "cloudflare":
        async for chunk in fetch_cloudflare_response_stream(client, url, headers, payload, model, timeout):
            yield chunk
    elif engine == "cohere":
        async for chunk in fetch_cohere_response_stream(client, url, headers, payload, model, timeout):
            yield chunk
    else:
        raise ValueError("Unknown response")



"""
A simplified, declarative implementation of the Beswarm worker agent system,
built using the custom MessageBroker for a high-level pub/sub architecture.
"""
import os
import sys
import uuid
import json
import copy
import asyncio
from pathlib import Path
from datetime import datetime
from typing import List, Dict, Union

from ..broker import MessageBroker
from ..bemcp.bemcp import MCPManager
from ..utils import register_mcp_tools
from ..aient.aient.models import chatgpt
from ..aient.aient.plugins import get_function_call_list, registry

worker_system_prompt = """
你是{name}。帮助用户头脑风暴。请分析不同用户的观点，并给出你的观点。
你的回答必须是 @用户名字开头。后面接上你的回复。如果你觉得无需回答的时候，请直接回复<stop>。
"""

class BaseAgent:
    """Base class for agents, handling common initialization and disposal."""
    def __init__(self, name: str, tools_json: List, agent_config: Dict, work_dir: str, cache_messages: Union[bool, List[Dict]], broker: MessageBroker, listen_topic: str, publish_topic: str, status_topic: str):

        self.id = str(uuid.uuid4())
        self.name = name
        self.tools_json = tools_json
        self.work_dir = work_dir
        self.cache_file = Path(work_dir) / ".beswarm" / "work_agent_conversation_history.json"
        self.config = copy.deepcopy(agent_config)
        self.cache_messages = cache_messages
        if cache_messages and isinstance(cache_messages, bool) and cache_messages == True:
            self.cache_messages = json.loads(self.cache_file.read_text(encoding="utf-8"))
        self.broker = broker
        self.listen_topic = listen_topic
        self.error_topic = listen_topic + ".error"
        self.publish_topic = publish_topic
        self.status_topic = status_topic
        self._subscription = self.broker.subscribe(self.handle_message, [self.listen_topic, self.error_topic])

    async def handle_message(self, message: Dict):
        """Process incoming messages. Must be implemented by subclasses."""
        raise NotImplementedError

    def dispose(self):
        """Cancels the subscription and cleans up resources."""
        if self._subscription:
            self._subscription.dispose()


class WorkerAgent(BaseAgent):
    """Executes instructions and publishes results to a message broker."""
    def __init__(self, name: str, tools_json: List, agent_config: Dict, work_dir: str, cache_messages: Union[bool, List[Dict]], broker: MessageBroker, listen_topic: str, publish_topic: str, status_topic: str):
        super().__init__(name, tools_json, agent_config, work_dir, cache_messages, broker, listen_topic, publish_topic, status_topic)

        self.config["system_prompt"] = self.config["system_prompt"].format(name=self.name)
        self.agent = chatgpt(**self.config)

    async def handle_message(self, message: Dict):
        """Receives an instruction, executes it, and publishes the response."""

        if message.get("id") == self.id:
            return

        instruction = message["result"]
        response = await self.agent.ask_async(instruction)

        if '<stop>' in response.strip():
            if response.replace('<stop>', '').strip() != '':
                self.broker.publish({"id": self.id, "status": "new_message", "result": f"{self.name}: {response.replace('<stop>', '')}\n\n"}, self.status_topic)
            return

        if response.strip() == '':
            print("\n❌ 工作智能体回复为空，请重新生成指令。")
            self.broker.publish(message, self.error_topic)
        else:
            self.broker.publish({"id": self.id, "status": "new_message", "result": f"{self.name}: {response}\n\n"}, self.status_topic)
            self.broker.publish({
                "id": self.id,
                "result": f"{self.name}: {response}"
            }, self.publish_topic)

class Tee:
    def __init__(self, *files):
        self.files = files

    def write(self, obj):
        for f in self.files:
            f.write(obj)
            f.flush()

    def flush(self):
        for f in self.files:
            f.flush()

class ChatGroupWorker:
    """The 'glue' class that orchestrates agents via a MessageBroker."""
    def __init__(self, tools: List[Union[str, Dict]], work_dir: str, cache_messages: Union[bool, List[Dict]] = None, broker: MessageBroker = None, mcp_manager: MCPManager = None, task_manager = None, kgm = None):
        self.tools = tools
        self.work_dir = Path(work_dir)
        self.cache_messages = cache_messages

        self.broker = broker
        self.mcp_manager = mcp_manager
        self.task_manager = task_manager
        self.kgm = kgm
        self.task_completion_event = asyncio.Event()
        self.final_result = None
        self._status_subscription = None
        self.agents = []
        self.setup()

        self.channel = self.broker.request_channel()
        self.WORKER_RESPONSE_TOPIC = self.channel + ".worker_responses"
        self.TASK_STATUS_TOPIC =self.channel +  ".task_status"

    def setup(self):
        cache_dir = self.work_dir / ".beswarm"
        cache_dir.mkdir(parents=True, exist_ok=True)
        self.task_manager.set_root_path(self.work_dir)
        self.kgm.set_root_path(self.work_dir)
        self.cache_file = cache_dir / "work_agent_conversation_history.json"
        if not self.cache_file.exists():
            self.cache_file.write_text("[]", encoding="utf-8")

        DEBUG = os.getenv("DEBUG", "false").lower() in ("true", "1", "t", "yes")
        if DEBUG:
            log_file = open(cache_dir / "history.log", "a", encoding="utf-8")
            log_file.write(f"========== {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} ==========\n")
            original_stdout = sys.stdout
            original_stderr = sys.stderr
            sys.stdout = Tee(original_stdout, log_file)
            sys.stderr = Tee(original_stderr, log_file)

    async def _configure_tools(self):
        mcp_list = [item for item in self.tools if isinstance(item, dict)]
        if mcp_list:
            for mcp_item in mcp_list:
                mcp_name, mcp_config = list(mcp_item.items())[0]
                await self.mcp_manager.add_server(mcp_name, mcp_config)
                client = self.mcp_manager.clients.get(mcp_name)
                await register_mcp_tools(client, registry)
            all_mcp_tools = await self.mcp_manager.get_all_tools()
            self.tools.extend([tool.name for tool in sum(all_mcp_tools.values(), [])])
        self.tools = [item for item in self.tools if not isinstance(item, dict)]
        if "task_complete" not in self.tools: self.tools.append("task_complete")
        self.tools_json = [value for _, value in get_function_call_list(self.tools).items()]

    def _task_status_subscriber(self, message: Dict):
        """Subscriber for task status changes."""
        if message.get("status") == "finished":
            self.final_result = message.get("result")
            self.task_completion_event.set()

        if message.get("status") == "error":
            raise Exception(message.get("result"))

        if message.get("status") == "new_message":
            print(message.get("result"))
            chat_window = open(self.work_dir / ".beswarm" / "chat_window.md", "a", encoding="utf-8")
            chat_window.write("# " + message.get("result") + "\n")

    def _setup_agents(self):
        worker_agent_config = {
            "api_key": os.getenv("API_KEY"), "api_url": os.getenv("BASE_URL"),
            "engine": os.getenv("FAST_MODEL") or os.getenv("MODEL"),
            "system_prompt": worker_system_prompt,
            "print_log": True, "temperature": 0.5, "function_call_max_loop": 100
        }

        chat_agent1 = WorkerAgent(
            name="数学家", tools_json=self.tools_json, agent_config=worker_agent_config, work_dir=self.work_dir, cache_messages=self.cache_messages,
            broker=self.broker, listen_topic=self.WORKER_RESPONSE_TOPIC,
            publish_topic=self.WORKER_RESPONSE_TOPIC, status_topic=self.TASK_STATUS_TOPIC
        )

        chat_agent2 = WorkerAgent(
            name="哲学家", tools_json=self.tools_json, agent_config=worker_agent_config, work_dir=self.work_dir, cache_messages=self.cache_messages,
            broker=self.broker, listen_topic=self.WORKER_RESPONSE_TOPIC,
            publish_topic=self.WORKER_RESPONSE_TOPIC, status_topic=self.TASK_STATUS_TOPIC
        )
        self.agents = [chat_agent1, chat_agent2]

    async def run(self):
        """Sets up subscriptions and starts the workflow, waiting for user input."""
        os.chdir(self.work_dir.absolute())
        await self._configure_tools()

        self._setup_agents()

        self._status_subscription = self.broker.subscribe(self._task_status_subscriber, self.TASK_STATUS_TOPIC)

        print("开启群聊, 请输入内容开始对话, 输入 'exit' 或 'quit' 结束:")
        loop = asyncio.get_running_loop()

        async def user_input_loop():
            while True:
                try:
                    line = await loop.run_in_executor(None, sys.stdin.readline)
                    line = line.strip()
                    if not line:
                        continue
                    if line.lower() in ['exit', 'quit']:
                        self.task_completion_event.set()
                        break

                    self.broker.publish({"id": "user", "result": f"小蓝: {line}"}, self.WORKER_RESPONSE_TOPIC)
                    self.broker.publish({"id": "user", "status": "new_message", "result": f"小蓝: {line}\n\n"}, self.TASK_STATUS_TOPIC)
                except asyncio.CancelledError:
                    break

        input_task = asyncio.create_task(user_input_loop())

        await self.task_completion_event.wait()

        if not input_task.done():
            input_task.cancel()

        self.cleanup()
        return self.final_result

    async def run_for_web(self):
        """Sets up agents and subscriptions for web-based interaction, then yields messages."""
        os.chdir(self.work_dir.absolute())
        await self._configure_tools()
        self._setup_agents()

        self._status_subscription = self.broker.subscribe(self._task_status_subscriber, self.TASK_STATUS_TOPIC)

        print("Web chat group is running. Waiting for messages...")
        await self.task_completion_event.wait()
        print("Web chat group finished.")
        self.cleanup()

    def cleanup(self):
        """Cleans up resources like agent subscriptions and MCP manager."""
        print("Cleaning up resources...")
        for agent in self.agents:
            agent.dispose()
        if self._status_subscription:
            self._status_subscription.dispose()

        # This needs to be async, so we run it in a new event loop if needed
        try:
            loop = asyncio.get_running_loop()
            if loop.is_running():
                loop.create_task(self.mcp_manager.cleanup())
            else:
                asyncio.run(self.mcp_manager.cleanup())
        except RuntimeError: # No running loop
            asyncio.run(self.mcp_manager.cleanup())

    async def stream_run(self):
        """Runs the workflow and yields status messages."""
        os.chdir(self.work_dir.absolute())
        await self._configure_tools()

        instruction_agent, worker_agent = self._setup_agents()

        try:
            async for message in self.broker.iter_topic(self.TASK_STATUS_TOPIC):
                if message.get("status") == "new_message":
                    yield message.get("result")
                elif message.get("status") == "finished":
                    yield message.get("result")
                    break
                elif message.get("status") == "error":
                    raise Exception(message.get("result"))
        finally:
            instruction_agent.dispose()
            worker_agent.dispose()
            await self.mcp_manager.cleanup()


import contextvars
from .broker import MessageBroker
from .bemcp.bemcp import MCPManager
from .taskmanager import TaskManager
from .knowledge_graph import KnowledgeGraphManager

"""
全局共享实例
"""

broker = MessageBroker()
mcp_manager = MCPManager()
kgm = KnowledgeGraphManager(broker=broker)
current_task_manager = contextvars.ContextVar('current_task_manager')
current_work_dir = contextvars.ContextVar('current_work_dir', default=None)

# 动态系统提示扩展：允许用户注册可实时刷新的提示片段（字符串或可调用）
_system_prompt_providers = contextvars.ContextVar('system_prompt_providers', default=[])

def register_system_prompt_provider(provider):
    """
    注册一个系统提示扩展。
    - provider 可以是:
      * str: 固定文本
      * callable: 无参函数，返回字符串（每次渲染都会调用，达到实时刷新）
    """
    providers = list(_system_prompt_providers.get())
    providers.append(provider)
    _system_prompt_providers.set(providers)

def clear_system_prompt_providers():
    """清空已注册的系统提示扩展（仅影响当前任务上下文）。"""
    _system_prompt_providers.set([])

def render_system_prompt_extensions():
    """
    把所有扩展拼接成单一字符串。
    - 对于可调用项，会在每次渲染时调用。
    """
    parts = []
    for p in _system_prompt_providers.get():
        try:
            val = p() if callable(p) else p
            parts.append("" if val is None else f"<{p.__name__}>" + str(val) + f"</{p.__name__}>")
        except Exception as e:
            parts.append(f"[system_prompt_provider_error: {e}]")
    # 用两个换行分隔，避免影响主提示结构
    return "\n\n".join([s for s in parts if s])

def get_task_manager():
    """Creates a new, isolated TaskManager instance."""
    return TaskManager()



import re
import sys
import logging
from pathlib import Path
import queue
import atexit
from logging.handlers import QueueHandler, QueueListener

def extract_xml_content(text, xml_tag):
    result = ''
    pattern = rf'^<{xml_tag}>([\D\d\s]+?)<\/{xml_tag}>$'
    match = re.search(pattern, text, re.MULTILINE)
    if match:
        result = match.group(1)
    if not result:
        return ''
    return result.strip()

def replace_xml_content(original_string: str, tag_name: str, replacement_content: str) -> str:
    """
    将指定XML标签内的内容替换为新内容。

    此函数使用正则表达式查找所有匹配的XML标签对（例如 `<tag>...</tag>`），
    并将其中的内容替换为 `replacement_content`。

    Args:
        original_string (str): 包含XML标记的原始字符串。
        tag_name (str): 要定位的XML标签的名称（不带尖括号）。
        replacement_content (str): 用于替换标签内部内容的新字符串。

    Returns:
        str: 返回内容已被替换的新字符串。如果未找到匹配的标签，则返回原始字符串。
    """
    pattern = f"<{tag_name}>.*?<\\/{tag_name}>"
    replacement = f"<{tag_name}>{replacement_content}</{tag_name}>"

    new_string = re.sub(pattern, replacement, original_string, flags=re.DOTALL)

    return new_string

import io
import base64
from .aient.aient.core.utils import get_image_message, get_text_message

async def get_current_screen_image_message(prompt):
    print("instruction agent 正在截取当前屏幕...")
    try:
        import pyautogui
        # 使用 pyautogui 截取屏幕，返回 PIL Image 对象
        screenshot = pyautogui.screenshot()
        # img_width, img_height = screenshot.size # 获取截图尺寸
        img_width, img_height = pyautogui.size()
        print(f"截图成功，尺寸: {img_width}x{img_height}")

        # 将 PIL Image 对象转换为 Base64 编码的 PNG 字符串
        buffered = io.BytesIO()
        screenshot.save(buffered, format="PNG")
        base64_encoded_image = base64.b64encode(buffered.getvalue()).decode("utf-8")
        IMAGE_MIME_TYPE = "image/png" # 截图格式为 PNG

    except ImportError:
        # Pillow 也是 pyautogui 的依赖，但以防万一单独处理
        print("\n❌ 请安装所需库: pip install Pillow pyautogui")
        return False
    except Exception as e:
        print(f"\n❌ 截取屏幕或处理图像时出错: {e}")
        return False

    engine_type = "gpt"
    message_list = []
    text_message = await get_text_message(prompt, engine_type)
    image_message = await get_image_message(f"data:{IMAGE_MIME_TYPE};base64," + base64_encoded_image, engine_type)
    message_list.append(text_message)
    message_list.append(image_message)
    return message_list

import asyncio
from .bemcp.bemcp import MCPClient, convert_tool_format

async def register_mcp_tools(mcp_client: MCPClient, registry: "Registry"):
    """
    Fetches tools from an MCP client, dynamically creates corresponding
    Python functions with correct signatures, and registers them using the
    provided registry instance.
    """
    try:
        mcp_tools = await mcp_client.list_tools()
        print(f"Found {len(mcp_tools)} tools on the MCP server.")
    except Exception as e:
        print(f"Error fetching tools from MCP server: {e}")
        return

    # Create a dummy file for inspect.getsource to work with exec
    # This helps make the created functions inspectable
    source_storage = {}

    for tool in mcp_tools:
        openai_spec = convert_tool_format(tool)
        func_spec = openai_spec.get('function', {})

        tool_name = func_spec.get('name')
        if not tool_name or not tool_name.isidentifier():
            print(f"Skipping tool with invalid name: {tool_name}")
            continue

        description = func_spec.get('description', 'No description available.')
        params_spec = func_spec.get('parameters', {})
        properties = params_spec.get('properties', {})
        required = set(params_spec.get('required', []))

        param_defs = []
        for name, spec in properties.items():
            if name in required:
                param_defs.append(name)
            else:
                # Add default value for optional parameters
                default = spec.get('default', 'None')
                param_defs.append(f"{name}={default}")

        params_str = ", ".join(param_defs)
        arg_names = list(properties.keys())
        # Create a dictionary of arguments to pass to call_tool
        args_dict_str = "{" + ", ".join(f"'{k}': {k}" for k in arg_names) + "}"

        # Dynamically generate the Python source code for the function
        func_source = f"""
async def {tool_name}({params_str}):
    \"\"\"{description}\"\"\"
    tool_args = {args_dict_str}

    # This function is a dynamically generated wrapper for an MCP tool.
    print(f"-> Calling MCP tool '{tool_name}' with args: {{tool_args}}")
    result = await mcp_client.call_tool('{tool_name}', tool_args)

    # Extract text content if available, otherwise return the raw result
    if hasattr(result, 'content') and isinstance(result.content, list) and result.content:
        if hasattr(result.content[0], 'text'):
            return result.content[0].text
    return result
"""
        # Store the source code so inspect.getsource can find it
        source_storage[tool_name] = func_source

        # Use a dictionary for the execution scope
        exec_scope = {'mcp_client': mcp_client, 'asyncio': asyncio}

        try:
            # Execute the source code to create the function object
            exec(func_source, exec_scope)
            generated_func = exec_scope[tool_name]

            # Monkey-patch getsourcelines to work with our storage
            # This makes our generated function fully compatible with the registry
            import inspect
            import linecache
            linecache.cache[f"<mcp_adapter/{tool_name}>"] = (
                len(func_source),
                None,
                [line + '\n' for line in func_source.splitlines()],
                f"<mcp_adapter/{tool_name}>",
            )
            generated_func.__code__ = generated_func.__code__.replace(co_filename=f"<mcp_adapter/{tool_name}>")

            # Register the newly created function
            registry.register(type="tool", name=tool_name)(generated_func)
            print(f"Successfully registered MCP tool: '{tool_name}'")

        except Exception as e:
            import traceback
            print(f"Failed to create or register function for tool '{tool_name}': {e}")
            traceback.print_exc()

# For asynchronous logging
_listeners = []
_atexit_registered = False

def _stop_all_listeners():
    for listener in _listeners:
        listener.stop()

def setup_logger(logger_name: str, log_file: Path):
    """
    用异步方式设置一个 logger，使其同时输出到文件和终端，避免IO阻塞。

    Args:
        logger_name (str): Logger 的唯一名称。
        log_file (Path): 日志文件的完整路径。

    Returns:
        logging.Logger: 配置好的 logger 实例。
    """
    global _atexit_registered

    # 1. 获取 Logger 实例
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.INFO) # 设置 logger 的最低处理级别为 INFO
    logger.propagate = False

    # 如果已经配置了异步 handler，直接返回
    if any(isinstance(h, QueueHandler) for h in logger.handlers):
        return logger

    # 如果有其他 handler，说明被其他方式配置过，也直接返回
    if logger.hasHandlers():
        return logger

    # 2. 创建用于 Listener 的 Handler（文件和终端）
    # 确保日志文件所在的目录存在
    log_file.parent.mkdir(parents=True, exist_ok=True)
    file_handler = logging.FileHandler(log_file, mode='a', encoding='utf-8')
    stream_handler = logging.StreamHandler(sys.stdout)

    # 3. 创建一个通用的 Formatter
    formatter = logging.Formatter('%(asctime)s - %(name)s - [%(levelname)s] - %(message)s')

    # 4. 为两个 Handler 设置 Formatter
    file_handler.setFormatter(formatter)
    stream_handler.setFormatter(formatter)

    # 5. 设置 Queue、QueueHandler 和 QueueListener
    log_queue = queue.Queue(-1)
    # listener 在后台线程中从 queue 读取日志，并分发给 file_handler 和 stream_handler
    listener = QueueListener(log_queue, file_handler, stream_handler, respect_handler_level=True)

    # queue_handler 是一个非阻塞的 handler，它把日志消息放到 queue 中
    queue_handler = QueueHandler(log_queue)

    # 6. 将 QueueHandler 添加到 Logger
    logger.addHandler(queue_handler)

    # 7. 启动 listener 并注册 atexit 钩子以确保程序退出时停止
    _listeners.append(listener)
    listener.start()

    if not _atexit_registered:
        atexit.register(_stop_all_listeners)
        _atexit_registered = True

    return logger

if __name__ == "__main__":
    print(extract_xml_content("<instructions>\n123</instructions>", "instructions"))

# python -m beswarm.utils


import os
import zipfile
import argparse
from tqdm import tqdm

def zip_beswarm_folders(source_dir, zip_filename):
    """
    Traverses the source directory, finds all files within any .beswarm folder,
    and adds them to a zip file, preserving the directory structure.

    :param source_dir: The directory to search for .beswarm folders.
    :param zip_filename: The name of the output zip file.
    """
    if not os.path.isdir(source_dir):
        print(f"Error: Source directory '{source_dir}' not found.")
        return

    source_dir = os.path.abspath(source_dir)

    files_to_add = []
    # Walk the entire directory tree to find all files.
    for root, dirs, files in os.walk(source_dir):
        for file in files:
            file_path = os.path.join(root, file)
            # A file should be added if its path contains a '.beswarm' directory component.
            if '.beswarm' in file_path.split(os.sep):
                files_to_add.append(file_path)

    if not files_to_add:
        print("No files found within any .beswarm directories.")
        return

    print(f"Found {len(files_to_add)} files in .beswarm directories to zip.")

    # Create the zip file and add all the collected files.
    with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for file_path in tqdm(files_to_add, desc="Zipping files", unit="file"):
            # arcname is the path inside the zip file.
            arcname = os.path.relpath(file_path, source_dir)
            zipf.write(file_path, arcname)

    print(f"\nSuccessfully created '{zip_filename}'")

def main():
    parser = argparse.ArgumentParser(description="Beswarm main command line interface.")
    subparsers = parser.add_subparsers(dest='command', help='Available commands', required=True)

    # Create the parser for the "debug" command
    parser_debug = subparsers.add_parser('debug', help='Package all .beswarm folders from a directory into a zip file.')
    parser_debug.add_argument(
        "source_directory",
        help="The source directory to search for .beswarm folders."
    )
    parser_debug.add_argument(
        "-o", "--output",
        default="beswarm_debug.zip",
        help="The name of the output zip file (default: beswarm_debug.zip)."
    )

    args = parser.parse_args()

    if args.command == 'debug':
        zip_beswarm_folders(args.source_directory, args.output)
    else:
        parser.print_help()

if __name__ == "__main__":
    main()



import os
import base64
import mimetypes
from .registry import register_tool
import io
from PIL import Image, UnidentifiedImageError

@register_tool()
def read_image(image_path: str):
    """
读取本地图片文件，将其转换为 Base64 编码，并返回包含 MIME 类型和完整数据的字符串。
此工具用于将图片内容加载到上下文中。

参数:
    image_path (str): 本地图片文件的路径。

返回:
    str: 成功时返回包含图片MIME类型和Base64编码数据的格式化字符串。
            失败时返回错误信息字符串。
    """
    original_max_pixels = Image.MAX_IMAGE_PIXELS
    try:
        # 暂时禁用解压炸弹检查，以允许打开非常大的图像进行缩放。
        # 这是安全的，因为我们控制代码，并且会立即对图像进行缩减。
        Image.MAX_IMAGE_PIXELS = None

        # 检查路径是否存在
        if not os.path.exists(image_path):
            return f"<tool_error>图片路径 '{image_path}' 不存在。</tool_error>"
        # 检查是否为文件
        if not os.path.isfile(image_path):
            return f"<tool_error>路径 '{image_path}' 不是一个有效的文件 (可能是一个目录)。</tool_error>"

        # 尝试猜测MIME类型
        mime_type, _ = mimetypes.guess_type(image_path) # encoding 变量通常不需要

        if not mime_type or not mime_type.startswith('image/'):
            # 如果mimetypes无法识别，或者不是图片类型
            return f"<tool_error>文件 '{image_path}' 的MIME类型无法识别为图片 (检测到: {mime_type})。请确保文件是常见的图片格式 (e.g., PNG, JPG, GIF, WEBP)。</tool_error>"

        max_dim = 3072
        with Image.open(image_path) as img:
            width, height = img.size

            target_img = img
            if width > max_dim or height > max_dim:
                if width > height:
                    new_width = max_dim
                    new_height = int(height * (max_dim / width))
                else:
                    new_height = max_dim
                    new_width = int(width * (max_dim / height))

                try:
                    resampling_filter = Image.Resampling.LANCZOS
                except AttributeError:
                    # 兼容旧版 Pillow
                    resampling_filter = Image.LANCZOS

                target_img = img.resize((new_width, new_height), resampling_filter)

            # 将处理后的图片保存到内存中的字节流
            img_byte_arr = io.BytesIO()
            # 保留原始图片格式以获得最佳兼容性，如果无法确定格式，默认为PNG
            img_format = img.format or 'PNG'
            target_img.save(img_byte_arr, format=img_format)
            image_data = img_byte_arr.getvalue()

        base64_encoded_data = base64.b64encode(image_data).decode('utf-8')

        return f"data:{mime_type};base64," + base64_encoded_data

    except UnidentifiedImageError:
        return f"<tool_error>无法识别的图片格式 '{image_path}'，文件可能已损坏或格式不受支持。</tool_error>"
    except FileNotFoundError:
        # 这个异常通常由 open() 抛出，如果 os.path.exists 通过但文件在读取前被删除
        # 或者路径检查逻辑未能完全覆盖所有情况 (理论上不应发生)
        return f"<tool_error>图片路径 '{image_path}' 未找到 (可能在检查后被删除或移动)。</tool_error>"
    except PermissionError:
        return f"<tool_error>没有权限访问图片路径 '{image_path}'。</tool_error>"
    except IOError as e: # 例如文件损坏无法读取，或磁盘问题
        return f"<tool_error>读取图片 '{image_path}' 时发生 I/O 错误: {e}</tool_error>"
    except Exception as e:
        return f"<tool_error>读取图片 '{image_path}' 时发生未知错误: {e}</tool_error>"
    finally:
        # 恢复原始限制以避免副作用。
        Image.MAX_IMAGE_PIXELS = original_max_pixels


from .tools import *
from .aient.aient.architext.architext import Messages, SystemMessage, UserMessage, AssistantMessage, ToolCalls, ToolResults, Texts, RoleMessage, Images, Files
from .core import register_system_prompt_provider, clear_system_prompt_providers



import httpx
import base64
import os # 用于处理 API 密钥
import asyncio
import json
import re
# python -m core.test.test_geminimask
from ..utils import get_image_message, safe_get

# --- 请替换为您的实际值 ---
MODEL_NAME = "gemini-2.5-flash-preview-04-17"
API_KEY = os.environ.get("GEMINI_API_KEY") # 从环境变量读取密钥
IMAGE_PATH = os.environ.get("IMAGE_PATH")
# --------------------------

# 检查 API 密钥是否存在
if not API_KEY:
    raise ValueError("请设置 GOOGLE_API_KEY 环境变量或在代码中提供 API 密钥。")

# 确定图片的 MIME 类型
# 您可以根据文件扩展名进行猜测，或者使用更可靠的库如 python-magic
if IMAGE_PATH.lower().endswith(".png"):
    IMAGE_MIME_TYPE = "image/png"
elif IMAGE_PATH.lower().endswith(".jpg") or IMAGE_PATH.lower().endswith(".jpeg"):
    IMAGE_MIME_TYPE = "image/jpeg"
# 添加其他您需要支持的图片类型
else:
    raise ValueError(f"不支持的图片格式: {IMAGE_PATH}")

# 读取图片文件并进行 Base64 编码
try:
    with open(IMAGE_PATH, "rb") as image_file:
        image_data = image_file.read()
        base64_encoded_image = base64.b64encode(image_data).decode("utf-8")
        # print(base64_encoded_image)
except FileNotFoundError:
    print(f"错误：找不到图片文件 '{IMAGE_PATH}'")
    exit()

# image_message = get_image_message(base64_encoded_image, "gemini")
image_message = asyncio.run(get_image_message(f"data:{IMAGE_MIME_TYPE};base64," + base64_encoded_image, "gemini"))

# 构建请求 URL
url = f"https://generativelanguage.googleapis.com/v1beta/models/{MODEL_NAME}:generateContent"

prompt = "Give the segmentation masks for the Search box. Output a JSON list of segmentation masks where each entry contains the 2D bounding box in \"box_2d\" and the mask in \"mask\"."
# 定义查询参数 (API Key)
params = {"key": API_KEY}

# 定义请求头
headers = {"Content-Type": "application/json"}

# 定义请求体 (JSON payload)
payload = {
    "contents": [
        {
            "parts": [
                {
                    "text": prompt
                },
                image_message
            ]
        }
    ],
    "generationConfig": {"thinkingConfig": {"thinkingBudget": 0}},
}

# 发送 POST 请求
try:
    with httpx.Client() as client:
        response = client.post(url, params=params, headers=headers, json=payload, timeout=60.0) # 增加超时时间
        response.raise_for_status() # 如果状态码不是 2xx，则抛出异常

    # 您可以在这里添加代码来解析 response.json() 并提取分割掩码
    text = safe_get(response.json(), "candidates", 0, "content", "parts", 0, "text")
    # print(text)
    # 例如: segmentation_masks = response.json().get('candidates', [{}])[0].get('content', {}).get('parts', [{}])[0].get('text')

except httpx.HTTPStatusError as exc:
    print(f"HTTP 错误发生: {exc.response.status_code} - {exc.response.text}")
except httpx.RequestError as exc:
    print(f"发送请求时出错: {exc}")
except Exception as e:
    print(f"发生意外错误: {e}")


regex_pattern = r'(\[\s*\{.*?\}\s*\])' # 匹配包含至少一个对象的数组

# 使用 re.search 查找第一个匹配项，re.DOTALL 使点号能匹配换行符
match = re.search(regex_pattern, text, re.DOTALL)

if match:
    # 提取匹配到的整个 JSON 数组字符串 (group 1 因为模式中有括号)
    json_string = match.group(1)

    try:
        # 使用 json.loads() 解析字符串
        parsed_data = json.loads(json_string)
        # 使用 json.dumps 美化打印输出
        print(json.dumps(parsed_data, indent=2, ensure_ascii=False))

        # 例如，获取第一个元素的 label
        if isinstance(parsed_data, list) and len(parsed_data) > 0:
            first_item = parsed_data[0]
            if isinstance(first_item, dict):
                label = first_item.get('label')
                print(f"\n第一个元素的 label 是: {label}")

    except json.JSONDecodeError as e:
        print(f"JSON 解析错误: {e}")
        print(f"出错的字符串是: {json_string}")
else:
    print("在文本中未找到匹配的 JSON 数组。")


import io
from PIL import Image, ImageDraw, ImageFont # pip install Pillow

def extract_box_and_mask_py(parsed_data):
    """
    从已解析的 JSON 数据中提取边界框 (box_2d) 和 Base64 编码的掩码 (mask) 数据。

    Args:
        parsed_data (list): 一个包含字典的列表，每个字典至少包含 'box_2d' 和 'mask' 键。
                            例如: [{'box_2d': [y1, x1, y2, x2], 'mask': 'data:image/png;base64,...'}, ...]

    Returns:
        list: 一个包含字典的列表，每个字典包含 'box' (坐标列表)
              和 'mask_base64' (Base64 字符串)。
              例如: [{'box': [y1, x1, y2, x2], 'mask_base64': '...'}, ...]
              坐标系假定为 0-1000 范围。
    """
    # 不再需要正则表达式
    results = []
    # 检查 parsed_data 是否为列表
    if not isinstance(parsed_data, list):
        print(f"Error: Input data is not a list. Received type: {type(parsed_data)}")
        return results

    for item in parsed_data:
        if not isinstance(item, dict):
            print(f"Skipping non-dictionary item in list: {item}")
            continue

        try:
            box = item.get('box_2d')
            mask_data_uri = item.get('mask')

            # 检查 'box_2d' 和 'mask' 是否存在且不为 None
            if box is None or mask_data_uri is None:
                print(f"Skipping item due to missing 'box_2d' or 'mask': {item}")
                continue

            # 从 mask 数据 URI 中提取 Base64 部分
            # 格式: "data:image/[^;]+;base64,..."
            if isinstance(mask_data_uri, str) and mask_data_uri.startswith('data:image/') and ';base64,' in mask_data_uri:
                mask_b64 = mask_data_uri.split(';base64,', 1)[1]
            else:
                print(f"Skipping item due to invalid mask format: {mask_data_uri}")
                continue

            # 验证 box 数据
            if isinstance(box, list) and len(box) == 4 and all(isinstance(n, int) for n in box):
                 results.append({"box": box, "mask_base64": mask_b64})
            else:
                 print(f"Skipping invalid box format: {box}")

        # 捕捉可能的 KeyError 或其他在字典访问/处理中发生的错误
        except Exception as e:
            print(f"Error processing item: {item}. Error: {e}")

    return results

def display_image_with_bounding_boxes_and_masks_py(
    original_image_path,
    box_and_mask_data,
    output_overlay_path="overlay_image.png",
    output_compare_dir="comparison_outputs"
):
    """
    在原始图像上绘制边界框和掩码，并生成裁剪区域与掩码的对比图。

    Args:
        original_image_path (str): 原始图像的文件路径。
        box_and_mask_data (list): extract_box_and_mask_py 的输出列表。
        output_overlay_path (str): 保存带有叠加效果的图像的路径。
        output_compare_dir (str): 保存对比图像的目录路径。
    """
    try:
        img_original = Image.open(original_image_path).convert("RGBA")
        img_width, img_height = img_original.size
    except FileNotFoundError:
        print(f"Error: Original image not found at {original_image_path}")
        return
    except Exception as e:
        print(f"Error opening original image: {e}")
        return

    # 创建一个副本用于绘制叠加效果
    img_overlay = img_original.copy()
    draw = ImageDraw.Draw(img_overlay, "RGBA") # 使用 RGBA 模式以支持透明度

    # 定义颜色列表
    colors_hex = ['#FF0000', '#00FF00', '#0000FF', '#FFFF00', '#FF00FF', '#00FFFF']
    # 将十六进制颜色转换为 RGBA 元组 (用于绘制)
    colors_rgba = []
    for hex_color in colors_hex:
        h = hex_color.lstrip('#')
        rgb = tuple(int(h[i:i+2], 16) for i in (0, 2, 4))
        colors_rgba.append(rgb + (255,)) # (R, G, B, Alpha) - 边框完全不透明

    # 创建输出目录（如果不存在）
    import os
    os.makedirs(output_compare_dir, exist_ok=True)

    print(f"Found {len(box_and_mask_data)} box/mask pairs to process.")

    for i, data in enumerate(box_and_mask_data):
        box_0_1000 = data['box'] # [ymin, xmin, ymax, xmax] in 0-1000 range
        mask_b64 = data['mask_base64']
        color_index = i % len(colors_rgba)
        outline_color = colors_rgba[color_index]
        # 叠加掩码时使用半透明颜色
        mask_fill_color = outline_color[:3] + (int(255 * 0.7),) # 70% Alpha

        # --- 1. 坐标转换与验证 ---
        # 将 0-1000 坐标转换为图像像素坐标 (left, top, right, bottom)
        # 假设 box 是 [ymin, xmin, ymax, xmax]
        try:
            ymin_norm, xmin_norm, ymax_norm, xmax_norm = [c / 1000.0 for c in box_0_1000]

            left   = int(xmin_norm * img_width)
            top    = int(ymin_norm * img_height)
            right  = int(xmax_norm * img_width)
            bottom = int(ymax_norm * img_height)

            # 确保坐标在图像范围内且有效
            left = max(0, left)
            top = max(0, top)
            right = min(img_width, right)
            bottom = min(img_height, bottom)

            box_width_px = right - left
            box_height_px = bottom - top

            if box_width_px <= 0 or box_height_px <= 0:
                print(f"Skipping box {i+1} due to zero or negative dimensions after conversion.")
                continue

        except Exception as e:
            print(f"Error processing coordinates for box {i+1}: {box_0_1000}. Error: {e}")
            continue

        print(f"Processing Box {i+1}: Pixels(L,T,R,B)=({left},{top},{right},{bottom}) Color={colors_hex[color_index]}")

        # --- 2. 在叠加图像上绘制边界框 ---
        try:
            draw.rectangle([left, top, right, bottom], outline=outline_color, width=5)
        except Exception as e:
             print(f"Error drawing rectangle for box {i+1}: {e}")
             continue

        # --- 3. 处理并绘制掩码 ---
        try:
            # 解码 Base64 掩码数据
            mask_bytes = base64.b64decode(mask_b64)
            mask_img_raw = Image.open(io.BytesIO(mask_bytes)).convert("RGBA")

            # 将掩码图像缩放到边界框的像素尺寸
            mask_img_resized = mask_img_raw.resize((box_width_px, box_height_px), Image.Resampling.NEAREST)

            # 创建一个纯色块，应用掩码的 alpha 通道
            color_block = Image.new('RGBA', mask_img_resized.size, mask_fill_color)

            # 将带有透明度的颜色块粘贴到叠加图像上，使用掩码的 alpha 通道作为粘贴蒙版
            # mask_img_resized.split()[-1] 提取 alpha 通道
            img_overlay.paste(color_block, (left, top), mask=mask_img_resized.split()[-1])

        except base64.binascii.Error:
             print(f"Error: Invalid Base64 data for mask {i+1}.")
             continue
        except Exception as e:
             print(f"Error processing or drawing mask for box {i+1}: {e}")
             continue

        # --- 4. 生成对比图 ---
        try:
            # 从原始图像中裁剪出边界框区域
            img_crop = img_original.crop((left, top, right, bottom))

            # 准备掩码预览图（使用原始解码后的掩码，调整大小以匹配裁剪区域）
            # 这里直接使用缩放后的 mask_img_resized 的 RGB 部分可能更直观
            mask_preview = mask_img_resized.convert("RGB") # 转换为 RGB 以便保存为常见格式

            # 保存裁剪图和掩码预览图
            crop_filename = os.path.join(output_compare_dir, f"compare_{i+1}_crop.png")
            mask_filename = os.path.join(output_compare_dir, f"compare_{i+1}_mask.png")
            img_crop.save(crop_filename)
            mask_preview.save(mask_filename)
            print(f" - Saved comparison: {crop_filename}, {mask_filename}")

        except Exception as e:
            print(f"Error creating or saving comparison images for box {i+1}: {e}")

    # --- 5. 保存最终的叠加图像 ---
    try:
        img_overlay.save(output_overlay_path)
        print(f"\nOverlay image saved to: {output_overlay_path}")
        print(f"Comparison images saved in: {output_compare_dir}")
    except Exception as e:
        print(f"Error saving the final overlay image: {e}")


extracted_data = extract_box_and_mask_py(parsed_data)

if extracted_data:
    # 确保原始图像存在
    import os
    if os.path.exists(IMAGE_PATH):
            display_image_with_bounding_boxes_and_masks_py(
                IMAGE_PATH,
                extracted_data,
                output_overlay_path="python_overlay_output.png", # 输出带叠加效果的图片名
                output_compare_dir="python_comparison_outputs" # 输出对比图的文件夹名
            )
    else:
            print(f"Error: Cannot proceed with visualization, image file not found: {IMAGE_PATH}")
            print("Please update the 'IMAGE_PATH' variable in the script.")
else:
    print("No valid box and mask data found in the response text.")



# server.py
import httpx
from mcp.server.fastmcp import FastMCP
from mcp.server.fastmcp.prompts import base

# Create an MCP server
mcp = FastMCP("Demo")


# Add an addition tool
@mcp.tool()
def add(a: int, b: int) -> int:
    """Add two numbers"""
    return a + b

@mcp.tool()
def calculate_bmi(weight_kg: float, height_m: float) -> float:
    """Calculate BMI given weight in kg and height in meters"""
    return weight_kg / (height_m**2)

@mcp.tool()
async def fetch_weather(city: str) -> str:
    """Fetch current weather for a city"""
    async with httpx.AsyncClient() as client:
        response = await client.get(f"https://api.weather.com/{city}")
        return response.text

@mcp.resource("config://app")
def get_config() -> str:
    """Static configuration data"""
    return "App configuration here"

@mcp.prompt()
def debug_error(error: str) -> list[base.Message]:
    return [
        base.UserMessage("I'm seeing this error:"),
        base.UserMessage(error),
        base.AssistantMessage("I'll help debug that. What have you tried so far?"),
    ]

if __name__ == "__main__":
    # mcp.run(transport='stdio')
    mcp.run(transport='sse')


"""
使用 Reaktiv 模拟消息队列 (发布/订阅)

本模块提供了一个 MessageBroker 类，它利用 Reaktiv 的核心原语（Signal, Computed, Effect）
来构建一个功能类似消息队列的、内存中的发布/订阅系统。
"""
import asyncio
from typing import Callable, Any, List, Union, Tuple

from reaktiv import Signal, Effect, Computed, untracked, to_async_iter

class Subscription:
    """封装一个或多个 Effect，提供统一的暂停、恢复和取消订阅的接口。"""

    def __init__(
        self,
        broker: "MessageBroker",
        callback: Callable[[Any], None],
        effects_with_topics: List[Tuple[Effect, str]],
    ):
        self._broker = broker
        self._callback = callback
        self._effects_with_topics = effects_with_topics
        self._effects = [e for e, t in effects_with_topics]
        self.is_paused = Signal(False)

    def pause(self):
        """暂停订阅，将不再处理新消息。"""
        self.is_paused.set(True)
        if self._broker.debug:
            print(f"Subscription paused.")

    def resume(self):
        """恢复订阅，将继续处理新消息。"""
        self.is_paused.set(False)
        if self._broker.debug:
            print(f"Subscription resumed.")

    def dispose(self):
        """永久取消订阅并清理资源。"""
        for effect, topic in self._effects_with_topics:
            effect.dispose()
            # 从代理的注册表中移除
            if (
                topic in self._broker._effects_registry
                and self._callback in self._broker._effects_registry[topic]
            ):
                del self._broker._effects_registry[topic][self._callback]
                if not self._broker._effects_registry[topic]:
                    del self._broker._effects_registry[topic]
        if self._broker.debug:
            print(f"Subscription disposed.")


class MessageBroker:
    """一个简单的消息代理，使用 Reaktiv Signal 和 Computed 模拟消息队列和派生主题。"""

    def __init__(self, debug: bool = False):
        # 现在 _topics 可以存储 Signal (原始主题) 或 Computed (派生主题)。
        self._topics: dict[str, Union[Signal[List[Any]], Computed[List[Any]]]] = {}
        # 新增: 注册表来跟踪 (主题, 回调) -> Effect 的映射
        self._effects_registry: dict[str, dict[Callable, Effect]] = {}
        self.debug = debug
        self._channel_counters: dict[str, int] = {}
        # print("消息代理已启动。")

    def request_channel(self, prefix: str = "channel") -> str:
        """
        申请一个新的、唯一的频道名称。

        此方法为每个前缀维护一个独立的计数器。
        返回一个基于前缀和该前缀当前计数值的唯一字符串，例如 'channel0', 'worker_0', 'channel1'。
        它不直接创建主题或任何关联的 Signal；这将在首次发布或订阅到返回的主题名称时发生。

        Args:
            prefix: 频道名称的前缀。默认为 'channel'。

        Returns:
            一个基于前缀的唯一主题/频道名称字符串。
        """
        if prefix not in self._channel_counters:
            self._channel_counters[prefix] = 0

        channel_name = f"{prefix}{self._channel_counters[prefix]}"
        self._channel_counters[prefix] += 1
        return channel_name

    def publish(self, message: Any, topic: Union[str, List[str]] = "default"):
        """
        向一个或多个主题发布一条新消息。
        """
        topics_to_publish = [topic] if isinstance(topic, str) else topic

        for t in topics_to_publish:
            # 只能向原始主题发布
            topic_signal = self._topics.get(t)
            if not isinstance(topic_signal, Signal):
                print(f"警告：主题 '{t}' 不存在或不是一个可发布的原始主题。正在创建...")
                topic_signal = Signal([])
                self._topics[t] = topic_signal

            # 通过 update 方法追加新消息来触发更新。
            # 必须创建一个新列表才能让 Reaktiv 检测到变化。
            topic_signal.update(lambda messages: messages + [message])
            if self.debug:
                print(f"新消息发布到 '{t}': \"{message}\"")

    def subscribe(self, callback: Callable[[Any], None], topic: Union[str, List[str]] = "default") -> Subscription:
        """
        订阅一个或多个主题。每当有新消息发布时，回调函数将被调用。
        此方法是幂等的：重复订阅同一个回调到同一个主题不会产生副作用。

        Args:
            callback: 处理消息的回调函数。
            topic: 要订阅的主题，可以是单个字符串或字符串列表。

        Returns:
            一个 Subscription 实例，用于管理订阅的生命周期（暂停、恢复、取消）。
        """
        topics_to_subscribe = [topic] if isinstance(topic, str) else topic
        created_effects_with_topics = []

        # 创建一个 Subscription 实例来管理所有相关的 effects
        # 它需要提前创建，以便 effect_factory 可以访问它的 is_paused 信号
        subscription = Subscription(self, callback, created_effects_with_topics)

        for t in topics_to_subscribe:
            # 检查此回调是否已订阅该主题
            if t in self._effects_registry and callback in self._effects_registry.get(t, {}):
                print(f"警告：订阅者 '{callback.__name__}' 已经订阅了 '{t}' 主题。跳过。")
                continue

            if t not in self._topics:
                # 如果订阅一个不存在的主题，也为它创建一个 Signal
                self._topics[t] = Signal([])

            # 使用一个工厂函数来为每个主题创建独立的闭包，
            # 确保每个订阅都有自己的 'last_processed_index'。
            def effect_factory(current_topic: str):
                last_processed_index = 0

                def process_new_messages():
                    nonlocal last_processed_index
                    all_messages = self._topics[current_topic]()

                    # 如果暂停了，只更新索引以跳过消息，不进行处理
                    if untracked(subscription.is_paused):
                        last_processed_index = len(all_messages)
                        return

                    new_messages = all_messages[last_processed_index:]

                    if new_messages:
                        if self.debug:
                            print(f"    -> 订阅者 '{callback.__name__}' 在 '{current_topic}' 主题上收到 {len(new_messages)} 条新消息。{new_messages}")
                        for msg in new_messages:
                            try:
                                if asyncio.iscoroutinefunction(callback):
                                    asyncio.create_task(callback(msg))
                                else:
                                    callback(msg)
                            except Exception as e:
                                print(f"    !! 在订阅者 '{callback.__name__}' 中发生错误: {e}")

                    last_processed_index = len(all_messages)
                return process_new_messages

            if self.debug:
                print(f"订阅者 '{callback.__name__}' 已订阅 '{t}' 主题。")
            effect = Effect(effect_factory(t))

            # 注册新的 effect
            if t not in self._effects_registry:
                self._effects_registry[t] = {}
            self._effects_registry[t][callback] = effect

            created_effects_with_topics.append((effect, t))

        return subscription

    def create_derived_topic(self, new_topic_name: str, source_topic: str, transform_fn: Callable[[List[Any]], List[Any]]):
        """
        创建一个派生主题。

        这个新主题的内容是一个 Computed 信号，它会根据源主题的内容和转换函数自动更新。

        Args:
            new_topic_name: 派生主题的名称。
            source_topic: 源主题的名称。
            transform_fn: 一个函数，接收源主题的消息列表并返回新的消息列表。
        """
        if new_topic_name in self._topics:
            print(f"警告：主题 '{new_topic_name}' 已存在。")
            return

        source_signal = self._topics.get(source_topic)
        if not isinstance(source_signal, (Signal, Computed)):
            print(f"错误：源主题 '{source_topic}' 不存在。")
            return

        # 创建一个 Computed 信号作为派生主题
        derived_signal = Computed(
            lambda: transform_fn(source_signal())
        )

        self._topics[new_topic_name] = derived_signal
        if self.debug:
            print(f"已从 '{source_topic}' 创建派生主题 '{new_topic_name}'。")

    async def iter_topic(self, topic: str):
        """
        返回一个异步迭代器，用于通过 async for 循环消费主题消息。

        Args:
            topic: 要订阅的主题名称。

        Yields:
            主题中的新消息。
        """
        if topic not in self._topics:
            # 如果主题不存在，创建一个，以防万一
            self._topics[topic] = Signal([])

        topic_signal = self._topics[topic]
        last_yielded_index = 0

        # to_async_iter 会在每次 topic_signal 更新时产生一个新的消息列表
        async for all_messages in to_async_iter(topic_signal):
            new_messages = all_messages[last_yielded_index:]
            for msg in new_messages:
                # 过滤掉内部的 'init' 消息
                if msg != "init":
                    yield msg

            last_yielded_index = len(all_messages)


import os
import re
import json
import copy
import httpx
import asyncio
import logging
import inspect
from collections import defaultdict
from typing import Union, Optional, Callable

from .base import BaseLLM
from ..plugins.registry import registry
from ..plugins import PLUGINS, get_tools_result_async, function_call_list, update_tools_config
from ..utils.scripts import safe_get, async_generator_to_sync, parse_function_xml, parse_continuous_json, convert_functions_to_xml, remove_xml_tags_and_content
from ..core.request import prepare_request_payload
from ..core.response import fetch_response_stream, fetch_response
from ..architext.architext import Messages, SystemMessage, UserMessage, AssistantMessage, ToolCalls, ToolResults, Texts, RoleMessage, Images, Files

class ToolResult(Texts):
    def __init__(self, tool_name: str, tool_args: str, tool_response: str, name: Optional[str] = None, visible: bool = True, newline: bool = True):
        super().__init__(text=tool_response, name=name or f"tool_result_{tool_name}", visible=visible, newline=newline)
        self.tool_name = tool_name
        self.tool_args = tool_args

    async def render(self) -> Optional[str]:
        tool_response = await super().render()
        if tool_response is None:
            tool_response = ""
        if self.tool_args:
            return f"[{self.tool_name}({self.tool_args}) Result]:\n\n{tool_response}"
        else:
            return f"[{self.tool_name} Result]:\n\n{tool_response}"

class APITimeoutError(Exception):
    """Custom exception for API timeout errors."""
    pass

class ValidationError(Exception):
    """Custom exception for response validation errors."""
    def __init__(self, message, response_text):
        super().__init__(message)
        self.response_text = response_text

class EmptyResponseError(Exception):
    """Custom exception for empty API responses."""
    pass

class ModelNotFoundError(Exception):
    """Custom exception for model not found (404) errors."""
    pass

class RateLimitError(Exception):
    """Custom exception for rate limit (429) errors."""
    pass

class BadRequestError(Exception):
    """Custom exception for bad request (400) errors."""
    pass

class HTTPError(Exception):
    """Custom exception for HTTP 500 errors."""
    pass

class InputTokenCountExceededError(Exception):
    """Custom exception for input token count exceeding the maximum."""
    pass

class ConfigurationError(Exception):
    """Custom exception for configuration errors."""
    pass

class RetryFailedError(Exception):
    """Custom exception for retry failures."""
    pass

class TaskComplete(Exception):
    """Exception-like signal to indicate the task is complete."""
    def __init__(self, message):
        self.completion_message = message
        super().__init__(f"Task completed with message: {message}")

# 结尾重复响应错误
class RepetitiveResponseError(Exception):
    """Custom exception for detecting repetitive and meaningless generated strings."""
    def __init__(self, message, phrase, count):
        super().__init__(message)
        self.phrase = phrase
        self.count = count


class AllToolsMissingParametersError(Exception):
    """Custom exception for when all tools are missing required parameters."""
    def __init__(self, message, response_text):
        super().__init__(message)
        self.response_text = response_text


class chatgpt(BaseLLM):
    """
    Official ChatGPT API
    """

    def __init__(
        self,
        api_key: str = None,
        engine: str = os.environ.get("MODEL") or "gpt-4o",
        api_url: str = (os.environ.get("BASE_URL") or "https://api.openai.com/v1/chat/completions"),
        system_prompt: str = "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally",
        proxy: str = None,
        timeout: float = 600,
        max_tokens: int = None,
        temperature: float = 0.5,
        top_p: float = 1.0,
        presence_penalty: float = 0.0,
        frequency_penalty: float = 0.0,
        reply_count: int = 1,
        truncate_limit: int = None,
        use_plugins: bool = True,
        print_log: bool = False,
        tools: Optional[Union[list, str, Callable]] = [],
        function_call_max_loop: int = 3,
        cache_messages: list = None,
        logger: logging.Logger = None,
        check_done: bool = False,
        retry_count: int = 999999,
    ) -> None:
        """
        Initialize Chatbot with API key (from https://platform.openai.com/account/api-keys)
        """
        super().__init__(api_key, engine, api_url, system_prompt, proxy, timeout, max_tokens, temperature, top_p, presence_penalty, frequency_penalty, reply_count, truncate_limit, use_plugins=use_plugins, print_log=print_log)
        self.conversation: dict[str, Messages] = defaultdict(lambda: Messages(SystemMessage(self.system_prompt)))
        if cache_messages:
            self.conversation["default"] = cache_messages
        self.function_calls_counter = {}
        self.function_call_max_loop = function_call_max_loop
        self.check_done = check_done
        self.retry_count = retry_count
        if logger:
            self.logger = logger
        else:
            # 如果没有提供 logger，创建一个默认的，它只会打印到控制台
            self.logger = logging.getLogger("chatgpt_default")
            self.logger.propagate = False
            if not self.logger.handlers: # 防止重复添加 handler
                handler = logging.StreamHandler()
                formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
                handler.setFormatter(formatter)
                self.logger.addHandler(handler)
                self.logger.setLevel(logging.INFO if print_log else logging.WARNING)

        # 注册和处理传入的工具
        self._register_tools(tools)


    def _register_tools(self, tools):
        """动态注册工具函数并更新配置"""

        self.plugins = copy.deepcopy(PLUGINS)
        self.function_call_list = copy.deepcopy(function_call_list)
        # 如果有新工具，需要注册到registry并更新配置
        self.plugins, self.function_call_list, _ = update_tools_config()

        if isinstance(tools, list):
            self.tools = tools if tools else []
        else:
            self.tools = [tools] if tools else []

        for tool in self.tools:
            tool_name = tool.__name__ if callable(tool) else str(tool)
            if tool_name in self.plugins:
                self.plugins[tool_name] = True
            else:
                raise ValueError(f"Tool {tool_name} not found in plugins")

    def add_to_conversation(
        self,
        message: Union[str, list],
        role: str,
        convo_id: str = "default",
        function_name: str = "",
        total_tokens: int = 0,
        function_arguments: str = "",
        pass_history: int = 9999,
        function_call_id: str = "",
    ) -> None:
        """
        Add a message to the conversation
        """
        # self.logger.info(f"role: {role}, function_name: {function_name}, message: {message}, function_arguments: {function_arguments}")
        if convo_id not in self.conversation:
            self.reset(convo_id=convo_id)
        if function_name == "" and message:
            self.conversation[convo_id].append(RoleMessage(role, message))
        elif function_name != "" and message:
            if not (all(value == False for value in self.plugins.values()) or self.use_plugins == False):
                tool_calls = [
                    {
                        "id": function_call_id,
                        "type": "function",
                        "function": {
                            "name": function_name,
                            "arguments": function_arguments,
                        },
                    }
                ]
                self.conversation[convo_id].append(ToolCalls(tool_calls))
                self.conversation[convo_id].append(ToolResults(tool_call_id=function_call_id, content=message))
            else:
                last_user_message = self.conversation[convo_id][-1]
                if last_user_message != UserMessage(message):
                    image_message_list = UserMessage()
                    if isinstance(function_arguments, str):
                        functions_list = json.loads(function_arguments)
                    else:
                        functions_list = function_arguments
                    for tool_info in functions_list:
                        if tool_info.get("base64_image"):
                            image_message_list.extend([
                                safe_get(tool_info, "parameter", "image_path", default="") + " image:",
                                Images(tool_info["base64_image"]),
                            ])
                    self.conversation[convo_id].append(AssistantMessage(convert_functions_to_xml(function_arguments)))
                    if image_message_list:
                        self.conversation[convo_id].append(UserMessage(message + image_message_list, Texts("\n\nYour message **must** end with [done] to signify the end of your output.", name="done", visible=self.check_done)))
                    else:
                        self.conversation[convo_id].append(UserMessage(message, Texts("\n\nYour message **must** end with [done] to signify the end of your output.", name="done", visible=self.check_done)))
                else:
                    self.conversation[convo_id].append(AssistantMessage("我已经执行过这个工具了，接下来我需要做什么？"))
        else:
            self.logger.error(f"error: add_to_conversation message is None or empty, role: {role}, function_name: {function_name}, message: {message}")

        # if self.print_log:
        #     replaced_text = json.loads(re.sub(r';base64,([A-Za-z0-9+/=]+)', ';base64,***', json.dumps(self.conversation[convo_id])))
        #     self.logger.info(json.dumps(replaced_text, indent=4, ensure_ascii=False))

        history_len = len(self.conversation[convo_id])

        history = pass_history
        if pass_history < 2:
            history = 2
        while history_len > history:
            mess_body = self.conversation[convo_id].pop(1)
            history_len = history_len - 1
            if mess_body.get("role") == "user":
                assistant_body = self.conversation[convo_id].pop(1)
                history_len = history_len - 1
                if assistant_body.get("tool_calls"):
                    self.conversation[convo_id].pop(1)
                    history_len = history_len - 1

        if total_tokens:
            self.current_tokens[convo_id] = total_tokens
            self.tokens_usage[convo_id] += total_tokens

    def truncate_conversation(self, convo_id: str = "default") -> None:
        """
        Truncate the conversation
        """
        while True:
            if (
                self.current_tokens[convo_id] > self.truncate_limit
                and len(self.conversation[convo_id]) > 1
            ):
                # Don't remove the first message
                mess = self.conversation[convo_id].pop(1)
                string_mess = json.dumps(mess, ensure_ascii=False)
                self.current_tokens[convo_id] -= len(string_mess) / 4
                self.logger.info(f"Truncate message: {mess}")
            else:
                break

    async def get_post_body(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = "",
        pass_history: int = 9999,
        stream: bool = True,
        **kwargs,
    ):
        # 构造 provider 信息
        provider = {
            "provider": "openai",
            "base_url": kwargs.get('api_url', self.api_url.chat_url),
            "api": kwargs.get('api_key', self.api_key),
            "model": [model or self.engine],
            "tools": True if self.use_plugins else False,
            "image": True
        }

        done_message = self.conversation[convo_id].provider("done")
        if done_message:
            done_message.visible = False
            if self.check_done and self.conversation[convo_id][-1][-1].name == "done":
                self.conversation[convo_id][-1][-1].visible = True

        # 构造请求数据
        request_data = {
            "model": model or self.engine,
            "messages": await self.conversation[convo_id].render_latest() if pass_history else Messages(
                SystemMessage(self.system_prompt, self.conversation[convo_id].provider("files")),
                UserMessage(prompt)
            ).render(),
            "stream": stream,
            "temperature": kwargs.get("temperature", self.temperature)
        }
        if stream:
            request_data["stream_options"] = {
                "include_usage": True
            }

        if kwargs.get("max_tokens", self.max_tokens):
            request_data["max_tokens"] = kwargs.get("max_tokens", self.max_tokens)

        # 添加工具相关信息
        if kwargs.get("plugins", None):
            self.plugins = kwargs.get("plugins")

        plugins_status = kwargs.get("plugins", self.plugins)
        if not (all(value == False for value in plugins_status.values()) or self.use_plugins == False):
            tools_request_body = []
            for item in plugins_status.keys():
                try:
                    if plugins_status[item]:
                        tools_request_body.append({"type": "function", "function": self.function_call_list[item]})
                except:
                    pass
            if tools_request_body:
                request_data["tools"] = tools_request_body
                request_data["tool_choice"] = "auto"

        # self.logger.info(f"request_data: {json.dumps(request_data, indent=4, ensure_ascii=False)}")

        # 调用核心模块的 prepare_request_payload 函数
        url, headers, json_post_body, engine_type = await prepare_request_payload(provider, request_data)

        return url, headers, json_post_body, engine_type

    async def _process_stream_response(
        self,
        response_gen,
        convo_id="default",
        function_name="",
        total_tokens=0,
        function_arguments="",
        function_call_id="",
        model="",
        language="English",
        system_prompt=None,
        pass_history=9999,
        is_async=False,
        stream: bool = True,
        **kwargs
    ):
        """
        处理流式响应的共用逻辑

        :param response_gen: 响应生成器(同步或异步)
        :param is_async: 是否使用异步模式
        """
        response_role = None
        full_response = ""
        function_full_response = ""
        function_call_name = ""
        need_function_call = False

        # 处理单行数据的公共逻辑
        def process_line(line):
            nonlocal response_role, full_response, function_full_response, function_call_name, need_function_call, total_tokens, function_call_id

            if not line or (isinstance(line, str) and line.startswith(':')):
                return None

            if isinstance(line, str) and line.startswith('data:'):
                line = line.lstrip("data: ")
                if line == "[DONE]":
                    return "DONE"
            elif isinstance(line, (dict, list)):
                if isinstance(line, dict) and safe_get(line, "choices", 0, "message", "content"):
                    full_response = line["choices"][0]["message"]["content"]
                    total_tokens = safe_get(line, "usage", "total_tokens", default=0)
                    return full_response
                else:
                    return str(line)
            else:
                try:
                    if isinstance(line, str):
                        line = json.loads(line)
                        if safe_get(line, "choices", 0, "message", "content"):
                            full_response = line["choices"][0]["message"]["content"]
                            return full_response
                        else:
                            return str(line)
                except:
                    self.logger.error(f"json.loads error: {repr(line)}")
                    return None

            resp = json.loads(line) if isinstance(line, str) else line
            if "error" in resp:
                raise Exception(json.dumps({"type": "api_error", "details": resp}, ensure_ascii=False))

            total_tokens = total_tokens or safe_get(resp, "usage", "total_tokens", default=0)
            delta = safe_get(resp, "choices", 0, "delta")
            if not delta:
                return None

            response_role = response_role or safe_get(delta, "role")
            if safe_get(delta, "content"):
                need_function_call = False
                content = delta["content"]
                full_response += content
                return content

            if safe_get(delta, "tool_calls"):
                need_function_call = True
                function_call_name = function_call_name or safe_get(delta, "tool_calls", 0, "function", "name")
                function_full_response += safe_get(delta, "tool_calls", 0, "function", "arguments", default="")
                function_call_id = function_call_id or safe_get(delta, "tool_calls", 0, "id")
                return None

        # 处理流式响应
        async def process_async():
            nonlocal response_role, full_response, function_full_response, function_call_name, need_function_call, total_tokens, function_call_id

            async for line in response_gen:
                line = line.strip() if isinstance(line, str) else line
                result = process_line(line)
                if result == "DONE":
                    break
                elif result:
                    yield result

        def process_sync():
            nonlocal response_role, full_response, function_full_response, function_call_name, need_function_call, total_tokens, function_call_id

            for line in response_gen:
                line = line.decode("utf-8") if hasattr(line, "decode") else line
                result = process_line(line)
                if result == "DONE":
                    break
                elif result:
                    yield result

        # 使用同步或异步处理器处理响应
        if is_async:
            async for chunk in process_async():
                yield chunk
        else:
            for chunk in process_sync():
                yield chunk

        if not full_response.strip() and not need_function_call:
            raise EmptyResponseError("Response is empty")

        if self.print_log:
            self.logger.info(f"total_tokens: {total_tokens}")

        if response_role is None:
            response_role = "assistant"

        missing_required_params = []

        if self.use_plugins == True:
            if self.check_done:
                # self.logger.info(f"worker Response: {full_response}")
                if not full_response.strip().endswith('[done]'):
                    raise ValidationError("Response is not ended with [done]", response_text=full_response)
                else:
                    full_response = full_response.strip().rstrip('[done]')
            full_response = full_response.replace("<tool_code>", "").replace("</tool_code>", "")
            function_parameter = parse_function_xml(full_response)
            if function_parameter:
                invalid_tools = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") not in self.plugins.keys()]
                function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") in self.plugins.keys()]

                # Check for missing required parameters
                valid_function_parameters = []
                for tool_dict in function_parameter:
                    tool_name = tool_dict.get("function_name")
                    # tool_name must be in registry.tools, because it is in self.plugins which is from registry.tools
                    func = registry.tools.get(tool_name)
                    if not func:
                        continue

                    sig = inspect.signature(func)
                    provided_params = tool_dict.get("parameter", {})
                    # Ensure provided_params is a dictionary
                    if not isinstance(provided_params, dict):
                        self.logger.warning(f"Parameters for {tool_name} are not a dict: {provided_params}. Skipping.")
                        continue

                    missing_required_params = []
                    for param in sig.parameters.values():
                        # Check if the parameter has no default value and is not in the provided parameters
                        if param.default is inspect.Parameter.empty and param.name not in provided_params:
                            missing_required_params.append(param.name)

                    if not missing_required_params:
                        valid_function_parameters.append(tool_dict)
                    else:
                        if self.print_log:
                            self.logger.warning(
                                f"Skipping tool call for '{tool_name}' due to missing required parameters: {missing_required_params}"
                            )
                            missing_required_params.append(f"Error: {tool_name} missing required parameters: {missing_required_params}")
                function_parameter = valid_function_parameters

                if not function_parameter and missing_required_params:
                    raise AllToolsMissingParametersError("\n\n".join(missing_required_params), response_text=full_response)

                # 删除 task_complete 跟其他工具一起调用的情况，因为 task_complete 必须单独调用
                if len(function_parameter) > 1:
                    function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "task_complete"]
                    # 仅当存在其他工具时，才删除 get_task_result
                    if any(tool.get("function_name") != "get_task_result" for tool in function_parameter):
                        function_parameter = [tool_dict for tool_dict in function_parameter if tool_dict.get("function_name", "") != "get_task_result"]
                if len(function_parameter) == 1 and function_parameter[0].get("function_name", "") == "task_complete":
                    raise TaskComplete(safe_get(function_parameter, 0, "parameter", "message", default="The task has been completed."))

                if self.print_log and invalid_tools:
                    self.logger.error(f"invalid_tools: {invalid_tools}")
                    self.logger.error(f"function_parameter: {function_parameter}")
                    self.logger.error(f"full_response: {full_response}")
                if function_parameter:
                    need_function_call = True
                    if isinstance(self.conversation[convo_id][-1]["content"], str) and \
                    "<tool_error>" in self.conversation[convo_id][-1]["content"]:
                        need_function_call = False
                        full_response = remove_xml_tags_and_content(full_response) + "上面是我的分析，还没有实际行动。\n\n接下来我需要做什么？"
                else:
                    need_function_call = False
                    if self.print_log:
                        self.logger.error(f"Failed to parse function_parameter full_response: {full_response}")
                    full_response = ""

        # 处理函数调用
        if need_function_call and self.use_plugins == True:
            if self.print_log:
                if function_parameter:
                    self.logger.info(f"function_parameter: {function_parameter}")
                else:
                    self.logger.info(f"function_full_response: {function_full_response}")

            function_response = ""
            # 定义处理单个工具调用的辅助函数
            async def process_single_tool_call(tool_name, tool_args, tool_id):
                nonlocal function_response

                if not self.function_calls_counter.get(tool_name):
                    self.function_calls_counter[tool_name] = 1
                else:
                    self.function_calls_counter[tool_name] += 1

                tool_response = ""
                has_args = safe_get(self.function_call_list, tool_name, "parameters", "required", default=False)
                if self.function_calls_counter[tool_name] <= self.function_call_max_loop and (tool_args != "{}" or not has_args):
                    if self.print_log:
                        self.logger.info(f"Tool use, calling: {tool_name}")

                    # 处理函数调用结果
                    if is_async:
                        async for chunk in get_tools_result_async(
                            tool_name, tool_args, model or self.engine, chatgpt, kwargs.get('api_key', self.api_key),
                            kwargs.get('api_url', self.api_url.chat_url), use_plugins=False, model=model or self.engine,
                            add_message=self.add_to_conversation, convo_id=convo_id, language=language
                        ):
                            yield chunk
                    else:
                        async def run_async():
                            async for chunk in get_tools_result_async(
                                tool_name, tool_args, model or self.engine, chatgpt, kwargs.get('api_key', self.api_key),
                                kwargs.get('api_url', self.api_url.chat_url), use_plugins=False, model=model or self.engine,
                                add_message=self.add_to_conversation, convo_id=convo_id, language=language
                            ):
                                yield chunk

                        for chunk in async_generator_to_sync(run_async()):
                            yield chunk
                else:
                    tool_response = f"无法找到相关信息，停止使用工具 {tool_name}"

                yield tool_response

            # 使用统一的JSON解析逻辑
            try:
                if function_full_response:
                    function_parameter = parse_continuous_json(function_full_response, function_call_name)
            except Exception as e:
                self.logger.error(f"解析JSON失败: {e}")
                # 保持原始工具调用
                tool_calls = [{
                    'function_name': function_call_name,
                    'parameter': function_full_response,
                    'function_call_id': function_call_id
                }]

            # 统一处理逻辑，将所有情况转换为列表处理
            if isinstance(function_parameter, list) and function_parameter:
                # 多个工具调用
                tool_calls = function_parameter

            # 处理所有工具调用
            all_responses = UserMessage()

            for tool_info in tool_calls:
                tool_name = tool_info['function_name']
                tool_args = json.dumps(tool_info['parameter'], ensure_ascii=False) if not isinstance(tool_info['parameter'], str) else tool_info['parameter']
                tool_id = tool_info.get('function_call_id', tool_name + "_tool_call")

                tool_response = ""
                if is_async:
                    async for chunk in process_single_tool_call(tool_name, tool_args, tool_id):
                        if isinstance(chunk, str) and "function_response:" in chunk:
                            tool_response = chunk.replace("function_response:", "")
                        else:
                            yield chunk
                else:
                    for chunk in async_generator_to_sync(process_single_tool_call(tool_name, tool_args, tool_id)):
                        if isinstance(chunk, str) and "function_response:" in chunk:
                            tool_response = chunk.replace("function_response:", "")
                        else:
                            yield chunk
                final_tool_response = tool_response
                if "<tool_error>" not in tool_response:
                    if tool_name == "read_file":
                        self.conversation[convo_id].provider("files").update(tool_info['parameter']["file_path"], tool_response, head=safe_get(tool_info, 'parameter', "head", default=None))
                        final_tool_response = "Read file successfully! The file content has been updated in the tag <latest_file_content>."
                    elif tool_name == "get_knowledge_graph_tree":
                        self.conversation[convo_id].provider("knowledge_graph").visible = True
                        final_tool_response = "Get knowledge graph tree successfully! The knowledge graph tree has been updated in the tag <knowledge_graph_tree>."
                    elif tool_name.endswith("goal"):
                        goal_provider = self.conversation[convo_id].provider("goal")
                        if goal_provider:
                            goal_provider += f"\n\n<{tool_name}>{tool_response}</{tool_name}>"
                        final_tool_response = "Get goal successfully! The goal has been updated in the tag <goal>."
                    elif tool_name == "write_to_file":
                        tool_args = None
                    elif tool_name == "read_image":
                        tool_info["base64_image"] = tool_response
                        final_tool_response = "Read image successfully!"
                    elif tool_response.startswith("data:image/") and ";base64," in tool_response:
                        tool_info["base64_image"] = tool_response
                        final_tool_response = "Read image successfully!"
                all_responses.append(ToolResult(tool_name, tool_args, final_tool_response))

            # 合并所有工具响应
            function_response = all_responses
            if missing_required_params:
                function_response.append(Texts("\n\n".join(missing_required_params)))

            # 使用第一个工具的名称和参数作为历史记录
            function_call_name = tool_calls[0]['function_name']
            function_full_response = function_full_response or json.dumps(tool_calls) if not isinstance(tool_calls[0]['parameter'], str) else tool_calls
            function_call_id = tool_calls[0].get('function_call_id', function_call_name + "_tool_call")

            response_role = "tool"

            # 递归处理函数调用响应
            if is_async:
                async for chunk in self.ask_stream_async(
                    function_response, response_role, convo_id=convo_id,
                    function_name=function_call_name, total_tokens=total_tokens,
                    model=model or self.engine, function_arguments=function_full_response,
                    function_call_id=function_call_id, api_key=kwargs.get('api_key', self.api_key),
                    api_url=kwargs.get('api_url', self.api_url.chat_url),
                    plugins=kwargs.get("plugins", self.plugins), system_prompt=system_prompt, stream=stream
                ):
                    yield chunk
            else:
                for chunk in self.ask_stream(
                    function_response, response_role, convo_id=convo_id,
                    function_name=function_call_name, total_tokens=total_tokens,
                    model=model or self.engine, function_arguments=function_full_response,
                    function_call_id=function_call_id, api_key=kwargs.get('api_key', self.api_key),
                    api_url=kwargs.get('api_url', self.api_url.chat_url),
                    plugins=kwargs.get("plugins", self.plugins), system_prompt=system_prompt, stream=stream
                ):
                    yield chunk
        else:
            # 添加响应到对话历史
            self.add_to_conversation(full_response, response_role, convo_id=convo_id, total_tokens=total_tokens, pass_history=pass_history)
            self.function_calls_counter = {}

    async def _ask_stream_handler(
        self,
        prompt: list,
        role: str = "user",
        convo_id: str = "default",
        model: str = "",
        pass_history: int = 9999,
        function_name: str = "",
        total_tokens: int = 0,
        function_arguments: str = "",
        function_call_id: str = "",
        language: str = "English",
        system_prompt: str = None,
        stream: bool = True,
        **kwargs,
    ):
        """
        Unified stream handler (async)
        """
        # 准备会话
        self.system_prompt = system_prompt or self.system_prompt
        if convo_id not in self.conversation or pass_history <= 2:
            self.reset(convo_id=convo_id, system_prompt=self.system_prompt)
        self.add_to_conversation(prompt, role, convo_id=convo_id, function_name=function_name, total_tokens=total_tokens, function_arguments=function_arguments, pass_history=pass_history, function_call_id=function_call_id)

        # 获取请求体
        url, headers, json_post, engine_type = await self.get_post_body(prompt, role, convo_id, model, pass_history, stream=stream, **kwargs)
        self.truncate_conversation(convo_id=convo_id)

        # 打印日志
        if self.print_log:
            self.logger.debug(f"api_url: {kwargs.get('api_url', self.api_url.chat_url)}")
            self.logger.debug(f"api_key: {kwargs.get('api_key', self.api_key)}")
        need_done_prompt = False

        # 发送请求并处理响应
        retry_times = 0
        error_to_raise = None
        while retry_times < self.retry_count:
            retry_times += 1
            tmp_post_json = copy.deepcopy(json_post)
            if need_done_prompt:
                tmp_post_json["messages"].extend(need_done_prompt)
            if self.print_log:
                replaced_text = json.loads(re.sub(r';base64,([A-Za-z0-9+/=]+)', ';base64,***', json.dumps(tmp_post_json)))
                replaced_text_str = json.dumps(replaced_text, indent=4, ensure_ascii=False)
                self.logger.info(f"Request Body:\n{replaced_text_str}")

            try:
                if prompt and "</" in prompt and "<instructions>" not in prompt and convert_functions_to_xml(parse_function_xml(prompt)).strip() == prompt:
                    tmp_response = {
                        "id": "chatcmpl-zXCi5TxWy953TCcxFocSienhvx0BB",
                        "object": "chat.completion.chunk",
                        "created": 1754588695,
                        "model": model or self.engine,
                        "choices": [
                            {
                                "index": 0,
                                "delta": {"role": "assistant", "content": prompt},
                                "finish_reason": "stop",
                            }
                        ],
                        "system_fingerprint": "fp_d576307f90",
                    }
                    async def _mock_response_generator():
                        yield f"data: {json.dumps(tmp_response)}\n\n"
                    generator = _mock_response_generator()
                else:
                    if stream:
                        generator = fetch_response_stream(
                            self.aclient, url, headers, tmp_post_json, engine_type, model or self.engine,
                        )
                    else:
                        generator = fetch_response(
                            self.aclient, url, headers, tmp_post_json, engine_type, model or self.engine,
                        )

                # 处理正常响应
                index = 0
                async for processed_chunk in self._process_stream_response(
                    generator, convo_id=convo_id, function_name=function_name,
                    total_tokens=total_tokens, function_arguments=function_arguments,
                    function_call_id=function_call_id, model=model, language=language,
                    system_prompt=system_prompt, pass_history=pass_history, is_async=True, stream=stream, **kwargs
                ):
                    if index == 0:
                        if "HTTP Error', 'status_code': 524" in processed_chunk:
                            raise APITimeoutError("Response timeout")
                        if "HTTP Error', 'status_code': 404" in processed_chunk:
                            raise ModelNotFoundError(f"Model: {model or self.engine} not found!")
                        if "HTTP Error', 'status_code': 429" in processed_chunk:
                            raise RateLimitError(f"Rate limit exceeded for model: {model or self.engine}")
                        if "HTTP Error', 'status_code': 413" in processed_chunk:
                            raise InputTokenCountExceededError(processed_chunk)
                        if "HTTP Error', 'status_code': 400" in processed_chunk:
                            raise BadRequestError(f"Bad Request: {processed_chunk}")
                        if "HTTP Error', 'status_code': " in processed_chunk:
                            raise HTTPError(f"HTTP Error: {processed_chunk}")
                    yield processed_chunk
                    index += 1

                # 成功处理，跳出重试循环
                break
            except (httpx.ConnectError, httpx.ReadTimeout, httpx.PoolTimeout):
                self.logger.error("Connection or read timeout.")
                return # Stop iteration
            except httpx.RemoteProtocolError:
                continue
            except httpx.ReadError as e:
                self.logger.warning(f"{e}, retrying...")
                continue
            except APITimeoutError:
                self.logger.warning("API response timeout (524), retrying...")
                continue
            except HTTPError as e:
                self.logger.warning(f"{e}, retrying...")
                continue
            except RateLimitError as e:
                self.logger.warning(f"{e}, retrying...")
                continue
            except InputTokenCountExceededError as e:
                self.logger.error(f"The request body is too long: {e}")
                error_to_raise = e
                break
            except BadRequestError as e:
                self.logger.error(f"Bad request error: {e}")
                raise
            except GeneratorExit:
                self.logger.warning("GeneratorExit caught, closing stream.")
                break
            except ValidationError as e:
                self.logger.warning(f"Validation failed: {e}. Retrying with corrective prompt.")
                need_done_prompt = [
                    {"role": "assistant", "content": e.response_text},
                    {"role": "user", "content": "你的消息没有以[done]结尾，请重新输出"}
                ]
                continue
            except AllToolsMissingParametersError as e:
                self.logger.warning(f"All tools are missing required parameters: {e}. Retrying with corrective prompt.")
                need_done_prompt = [
                    {"role": "assistant", "content": e.response_text},
                    {"role": "user", "content": f"{str(e)}，请重新输出"}
                ]
                continue
            except EmptyResponseError as e:
                self.logger.warning(f"{e}, retrying...")
                continue
            except RepetitiveResponseError as e:
                self.logger.warning(f"{e}, retrying...")
                continue
            except TaskComplete as e:
                raise
            except ModelNotFoundError as e:
                raise
            except Exception as e:
                self.logger.error(f"{e}")
                import traceback
                self.logger.error(traceback.format_exc())
                if "Invalid URL" in str(e):
                    error_message = "您输入了无效的API URL，请使用正确的URL并使用`/start`命令重新设置API URL。具体错误如下：\n\n" + str(e)
                    raise ConfigurationError(error_message)
                # 最后一次重试失败，向上抛出异常
                if retry_times == self.retry_count:
                    raise RetryFailedError(str(e))

        if error_to_raise:
            raise error_to_raise

    def ask_stream(
        self,
        prompt: list,
        role: str = "user",
        convo_id: str = "default",
        model: str = "",
        pass_history: int = 9999,
        function_name: str = "",
        total_tokens: int = 0,
        function_arguments: str = "",
        function_call_id: str = "",
        language: str = "English",
        system_prompt: str = None,
        stream: bool = True,
        **kwargs,
    ):
        """
        Ask a question (同步流式响应)
        """
        try:
            loop = asyncio.get_event_loop()
            if loop.is_closed():
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
        except RuntimeError:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)

        async_gen = self._ask_stream_handler(
            prompt, role, convo_id, model, pass_history, function_name, total_tokens,
            function_arguments, function_call_id, language, system_prompt, stream, **kwargs
        )
        for chunk in async_generator_to_sync(async_gen):
            yield chunk

    async def ask_stream_async(
        self,
        prompt: list,
        role: str = "user",
        convo_id: str = "default",
        model: str = "",
        pass_history: int = 9999,
        function_name: str = "",
        total_tokens: int = 0,
        function_arguments: str = "",
        function_call_id: str = "",
        language: str = "English",
        system_prompt: str = None,
        stream: bool = True,
        **kwargs,
    ):
        """
        Ask a question (异步流式响应)
        """
        async for chunk in self._ask_stream_handler(
            prompt, role, convo_id, model, pass_history, function_name, total_tokens,
            function_arguments, function_call_id, language, system_prompt, stream, **kwargs
        ):
            yield chunk

    async def ask_async(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = "",
        pass_history: int = 9999,
        **kwargs,
    ) -> str:
        """
        Non-streaming ask
        """
        response = self.ask_stream_async(
            prompt=prompt,
            role=role,
            convo_id=convo_id,
            pass_history=pass_history,
            model=model or self.engine,
            stream=True,
            **kwargs,
        )
        full_response: str = "".join([r async for r in response])
        return full_response

    def ask(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        model: str = "",
        pass_history: int = 9999,
        **kwargs,
    ) -> str:
        """
        Non-streaming ask
        """
        response = self.ask_stream(
            prompt=prompt,
            role=role,
            convo_id=convo_id,
            pass_history=pass_history,
            model=model or self.engine,
            stream=True,
            **kwargs,
        )
        full_response: str = "".join([r for r in response])
        return full_response

    def rollback(self, n: int = 1, convo_id: str = "default") -> None:
        """
        Rollback the conversation
        """
        for _ in range(n):
            self.conversation[convo_id].pop()

    def reset(self, convo_id: str = "default", system_prompt: str = "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally") -> None:
        """
        Reset the conversation
        """
        self.system_prompt = system_prompt or self.system_prompt
        self.conversation[convo_id] = Messages(
            SystemMessage(self.system_prompt, self.conversation[convo_id].provider("files")),
        )
        self.tokens_usage[convo_id] = 0
        self.current_tokens[convo_id] = 0
